@@ -21,18 +21,15 @@ help() {
   echo 'Usage:'
   echo '  pvscheck.sh [--pvs URL] [--deps] [--environment-cc]'
   echo '              [target-directory [branch]]'
-  echo '  pvscheck.sh [--pvs URL] [--recheck] [--environment-cc]'
+  echo '  pvscheck.sh [--pvs URL] [--recheck] [--environment-cc] [--update]'
   echo '              [target-directory]'
   echo '  pvscheck.sh [--pvs URL] --only-analyse [target-directory]'
   echo '  pvscheck.sh [--pvs URL] --pvs-install {target-directory}'
   echo '  pvscheck.sh --patch [--only-build]'
   echo
-  echo '    --pvs: Use the specified URL as a path to pvs-studio archive.'
-  echo '           By default latest tested version is used.'
+  echo '    --pvs: Fetch pvs-studio from URL.'
   echo
-  echo '           May use "--pvs detect" to try detecting latest version.'
-  echo '           That assumes certain viva64.com site properties and'
-  echo '           may be broken by the site update.'
+  echo '    --pvs detect: Auto-detect latest version (by scraping viva64.com).'
   echo
   echo '    --deps: (for regular run) Use top-level Makefile and build deps.'
   echo '            Without this it assumes all dependencies are already'
@@ -51,6 +48,8 @@ help() {
   echo
   echo '    --recheck: run analysis on a prepared target directory.'
   echo
+  echo '    --update: when rechecking first do a pull.'
+  echo
   echo '    --only-analyse: run analysis on a prepared target directory '
   echo '                    without building Neovim.'
   echo
@@ -306,8 +305,16 @@ create_compile_commands() {(
 # realpath is not available in Ubuntu trusty yet.
 realdir() {(
   local dir="$1"
-  cd "$dir"
-  printf '%s\n' "$PWD"
+  local add=""
+  while ! cd "$dir" 2>/dev/null ; do
+    add="${dir##*/}/$add"
+    local new_dir="${dir%/*}"
+    if test "$new_dir" = "$dir" ; then
+      return 1
+    fi
+    dir="$new_dir"
+  done
+  printf '%s\n' "$PWD/$add"
 )}
 
 patch_sources() {(
@@ -347,19 +354,35 @@ run_analysis() {(
 
   cd "$tgt"
 
+  # pvs-studio-analyzer exits with a non-zero exit code when there are detected
+  # errors, so ignore its return
   pvs-studio-analyzer \
     analyze \
       --threads "$(get_jobs_num)" \
       --output-file PVS-studio.log \
       --verbose \
       --file build/compile_commands.json \
-      --sourcetree-root .
+      --sourcetree-root . || true
 
-  plog-converter -t xml -o PVS-studio.xml PVS-studio.log
-  plog-converter -t errorfile -o PVS-studio.err PVS-studio.log
-  plog-converter -t tasklist -o PVS-studio.tsk PVS-studio.log
+  rm -rf PVS-studio.{xml,err,tsk,html.d}
+  local plog_args="PVS-studio.log --srcRoot . --excludedCodes V011"
+  plog-converter $plog_args --renderTypes xml       --output PVS-studio.xml
+  plog-converter $plog_args --renderTypes errorfile --output PVS-studio.err
+  plog-converter $plog_args --renderTypes tasklist  --output PVS-studio.tsk
+  plog-converter $plog_args --renderTypes fullhtml  --output PVS-studio.html.d
 )}
 
+detect_url() {
+  local url="${1:-detect}"
+  if test "$url" = detect ; then
+    curl --silent -L 'https://www.viva64.com/en/pvs-studio-download-linux/' \
+    | grep -o 'https\{0,1\}://[^"<>]\{1,\}/pvs-studio[^/"<>]*\.tgz' \
+    || echo FAILED
+  else
+    printf '%s' "$url"
+  fi
+}
+
 do_check() {
   local tgt="$1" ; shift
   local branch="$1" ; shift
@@ -367,17 +390,37 @@ do_check() {
   local deps="$1" ; shift
   local environment_cc="$1" ; shift
 
+  if test -z "$pvs_url" || test "$pvs_url" = FAILED ; then
+    pvs_url="$(detect_url detect)"
+    if test -z "$pvs_url" || test "$pvs_url" = FAILED ; then
+      echo "failed to auto-detect PVS URL"
+      exit 1
+    fi
+    echo "Auto-detected PVS URL: ${pvs_url}"
+  fi
+
   git clone --branch="$branch" . "$tgt"
 
   install_pvs "$tgt" "$pvs_url"
 
-  do_recheck "$tgt" "$deps" "$environment_cc"
+  do_recheck "$tgt" "$deps" "$environment_cc" ""
 }
 
 do_recheck() {
   local tgt="$1" ; shift
   local deps="$1" ; shift
   local environment_cc="$1" ; shift
+  local update="$1" ; shift
+
+  if test -n "$update" ; then
+    (
+      cd "$tgt"
+      local branch="$(git rev-parse --abbrev-ref HEAD)"
+      git checkout --detach
+      git fetch -f origin "${branch}:${branch}"
+      git checkout -f "$branch"
+    )
+  fi
 
   create_compile_commands "$tgt" "$deps" "$environment_cc"
 
@@ -397,22 +440,11 @@ do_analysis() {
   run_analysis "$tgt"
 }
 
-detect_url() {
-  local url="${1:-detect}"
-  if test "$url" = detect ; then
-    curl -L 'https://www.viva64.com/en/pvs-studio-download-linux/' \
-    | grep -o 'https\{0,1\}://[^"<>]\{1,\}/pvs-studio[^/"<>]*\.tgz'
-  else
-    printf '%s' "$url"
-  fi
-}
-
 main() {
-  local def_pvs_url="http://files.viva64.com/pvs-studio-6.15.21741.1-x86_64.tgz"
   eval "$(
     getopts_long \
       help store_const \
-      pvs 'modify detect_url pvs_url "${def_pvs_url}"' \
+      pvs 'modify detect_url pvs_url' \
       patch store_const \
       only-build 'store_const --only-build' \
       recheck store_const \
@@ -420,6 +452,7 @@ main() {
       pvs-install store_const \
       deps store_const \
       environment-cc store_const \
+      update store_const \
       -- \
       'modify realdir tgt "$PWD/../neovim-pvs"' \
       'store branch master' \
@@ -431,14 +464,14 @@ main() {
     return 0
   fi
 
-  set -x
+  # set -x
 
   if test -n "$patch" ; then
     patch_sources "$tgt" "$only_build"
   elif test -n "$pvs_install" ; then
     install_pvs "$tgt" "$pvs_url"
   elif test -n "$recheck" ; then
-    do_recheck "$tgt" "$deps" "$environment_cc"
+    do_recheck "$tgt" "$deps" "$environment_cc" "$update"
   elif test -n "$only_analyse" ; then
     do_analysis "$tgt"
   else
