@@ -38,7 +38,10 @@ typedef struct {
 #include "nvim/api/private/defs.h"
 // for Map(K, V)
 #include "nvim/map.h"
+// for kvec
+#include "nvim/lib/kvec.h"
 
+#define GETFILE_SUCCESS(x)    ((x) <= 0)
 #define MODIFIABLE(buf) (buf->b_p_ma)
 
 /*
@@ -55,10 +58,10 @@ typedef struct {
  * functions that set or reset the flags.
  *
  * VALID_BOTLINE    VALID_BOTLINE_AP
- *     on		on		w_botline valid
- *     off		on		w_botline approximated
- *     off		off		w_botline not valid
- *     on		off		not possible
+ *     on       on      w_botline valid
+ *     off      on      w_botline approximated
+ *     off      off     w_botline not valid
+ *     on       off     not possible
  */
 #define VALID_WROW      0x01    /* w_wrow (window row) is valid */
 #define VALID_WCOL      0x02    /* w_wcol (window col) is valid */
@@ -91,6 +94,7 @@ typedef struct {
 typedef struct window_S win_T;
 typedef struct wininfo_S wininfo_T;
 typedef struct frame_S frame_T;
+typedef uint16_t disptick_T;  // display tick type
 
 // for struct memline (it needs memfile_T)
 #include "nvim/memline_defs.h"
@@ -140,6 +144,12 @@ struct buffheader {
   size_t bh_space;          // space in bh_curr for appending
 };
 
+typedef struct
+{
+  buffheader_T sr_redobuff;
+  buffheader_T sr_old_redobuff;
+} save_redo_T;
+
 /*
  * Structure that contains all options that are local to a window.
  * Used twice in a window: for the current buffer and for all buffers.
@@ -149,7 +159,7 @@ typedef struct {
   int wo_arab;
 # define w_p_arab w_onebuf_opt.wo_arab  /* 'arabic' */
   int wo_bri;
-# define w_p_bri w_onebuf_opt.wo_bri	/* 'breakindent' */
+# define w_p_bri w_onebuf_opt.wo_bri    // 'breakindent'
   char_u *wo_briopt;
 # define w_p_briopt w_onebuf_opt.wo_briopt /* 'breakindentopt' */
   int wo_diff;
@@ -237,7 +247,7 @@ typedef struct {
   char_u *wo_winhl;
 # define w_p_winhl w_onebuf_opt.wo_winhl    // 'winhighlight'
 
-  int wo_scriptID[WV_COUNT];            /* SIDs for window-local options */
+  LastSet wo_scriptID[WV_COUNT];        // SIDs for window-local options
 # define w_p_scriptID w_onebuf_opt.wo_scriptID
 } winopt_T;
 
@@ -402,13 +412,13 @@ typedef struct {
    * syntax state too often.
    * b_sst_array[] is allocated to hold the state for all displayed lines,
    * and states for 1 out of about 20 other lines.
-   * b_sst_array	pointer to an array of synstate_T
-   * b_sst_len	number of entries in b_sst_array[]
-   * b_sst_first	pointer to first used entry in b_sst_array[] or NULL
-   * b_sst_firstfree	pointer to first free entry in b_sst_array[] or NULL
-   * b_sst_freecount	number of free entries in b_sst_array[]
-   * b_sst_check_lnum	entries after this lnum need to be checked for
-   *			validity (MAXLNUM means no check needed)
+   * b_sst_array        pointer to an array of synstate_T
+   * b_sst_len          number of entries in b_sst_array[]
+   * b_sst_first        pointer to first used entry in b_sst_array[] or NULL
+   * b_sst_firstfree    pointer to first free entry in b_sst_array[] or NULL
+   * b_sst_freecount    number of free entries in b_sst_array[]
+   * b_sst_check_lnum   entries after this lnum need to be checked for
+   *                    validity (MAXLNUM means no check needed)
    */
   synstate_T  *b_sst_array;
   int b_sst_len;
@@ -416,7 +426,7 @@ typedef struct {
   synstate_T  *b_sst_firstfree;
   int b_sst_freecount;
   linenr_T b_sst_check_lnum;
-  uint16_t b_sst_lasttick;      /* last display tick */
+  disptick_T b_sst_lasttick;    // last display tick
 
   // for spell checking
   garray_T b_langp;             // list of pointers to slang_T, see spell.c
@@ -465,24 +475,31 @@ struct file_buffer {
   int b_locked;                 // Buffer is being closed or referenced, don't
                                 // let autocommands wipe it out.
 
-  /*
-   * b_ffname has the full path of the file (NULL for no name).
-   * b_sfname is the name as the user typed it (or NULL).
-   * b_fname is the same as b_sfname, unless ":cd" has been done,
-   *		then it is the same as b_ffname (NULL for no name).
-   */
-  char_u      *b_ffname;        /* full path file name */
-  char_u      *b_sfname;        /* short file name */
-  char_u      *b_fname;         /* current file name */
+  //
+  // b_ffname   has the full path of the file (NULL for no name).
+  // b_sfname   is the name as the user typed it (or NULL).
+  // b_fname    is the same as b_sfname, unless ":cd" has been done,
+  //            then it is the same as b_ffname (NULL for no name).
+  //
+  char_u      *b_ffname;        // full path file name
+  char_u      *b_sfname;        // short file name
+  char_u      *b_fname;         // current file name
 
   bool file_id_valid;
   FileID file_id;
 
   int b_changed;                // 'modified': Set to true if something in the
                                 // file has been changed and not written out.
-/// Change identifier incremented for each change, including undo
-#define b_changedtick changedtick_di.di_tv.vval.v_number
-  ChangedtickDictItem changedtick_di;  // b:changedtick dictionary item.
+
+  /// Change identifier incremented for each change, including undo
+  ///
+  /// This is a dictionary item used to store in b:changedtick.
+  ChangedtickDictItem changedtick_di;
+
+  varnumber_T b_last_changedtick;       // b:changedtick when TextChanged or
+                                        // TextChangedI was last triggered.
+  varnumber_T b_last_changedtick_pum;   // b:changedtick when TextChangedP was
+                                        // last triggered.
 
   bool b_saving;                /* Set to true if we are in the middle of
                                    saving the buffer. */
@@ -570,12 +587,12 @@ struct file_buffer {
 
   bool b_scanned;               /* ^N/^P have scanned this buffer */
 
-  /* flags for use of ":lmap" and IM control */
-  long b_p_iminsert;            /* input mode for insert */
-  long b_p_imsearch;            /* input mode for search */
-#define B_IMODE_USE_INSERT -1   /*	Use b_p_iminsert value for search */
-#define B_IMODE_NONE 0          /*	Input via none */
-#define B_IMODE_LMAP 1          /*	Input via langmap */
+  // flags for use of ":lmap" and IM control
+  long b_p_iminsert;            // input mode for insert
+  long b_p_imsearch;            // input mode for search
+#define B_IMODE_USE_INSERT -1   //  Use b_p_iminsert value for search
+#define B_IMODE_NONE 0          //  Input via none
+#define B_IMODE_LMAP 1          //  Input via langmap
 # define B_IMODE_LAST 1
 
   short b_kmap_state;           /* using "lmap" mappings */
@@ -590,7 +607,7 @@ struct file_buffer {
    */
   bool b_p_initialized;                 /* set when options initialized */
 
-  int b_p_scriptID[BV_COUNT];           /* SIDs for buffer-local options */
+  LastSet b_p_scriptID[BV_COUNT];       // SIDs for buffer-local options
 
   int b_p_ai;                   ///< 'autoindent'
   int b_p_ai_nopaste;           ///< b_p_ai saved for paste mode
@@ -603,6 +620,7 @@ struct file_buffer {
   char_u *b_p_bt;               ///< 'buftype'
   int b_has_qf_entry;           ///< quickfix exists for buffer
   int b_p_bl;                   ///< 'buflisted'
+  long b_p_channel;             ///< 'channel'
   int b_p_cin;                  ///< 'cindent'
   char_u *b_p_cino;             ///< 'cinoptions'
   char_u *b_p_cink;             ///< 'cinkeys'
@@ -636,6 +654,7 @@ struct file_buffer {
   uint32_t b_p_fex_flags;       ///< flags for 'formatexpr'
   char_u *b_p_kp;               ///< 'keywordprg'
   int b_p_lisp;                 ///< 'lisp'
+  char_u *b_p_menc;             ///< 'makeencoding'
   char_u *b_p_mps;              ///< 'matchpairs'
   int b_p_ml;                   ///< 'modeline'
   int b_p_ml_nobin;             ///< b_p_ml saved for binary mode
@@ -716,6 +735,7 @@ struct file_buffer {
   int b_ind_hash_comment;
   int b_ind_cpp_namespace;
   int b_ind_if_for_while;
+  int b_ind_cpp_extern_c;
 
   linenr_T b_no_eol_lnum;       /* non-zero lnum when last line of next binary
                                  * write should not have an end-of-line */
@@ -740,7 +760,7 @@ struct file_buffer {
   /* Two special kinds of buffers:
    * help buffer  - used for help files, won't use a swap file.
    * spell buffer - used for spell info, never displayed and doesn't have a
-   *		      file name.
+   *                file name.
    */
   bool b_help;                  /* TRUE for help file buffer (when set b_p_bt
                                    is "help") */
@@ -763,6 +783,12 @@ struct file_buffer {
   BufhlInfo b_bufhl_info;       // buffer stored highlights
 
   kvec_t(BufhlLine *) b_bufhl_move_space;  // temporary space for highlights
+
+  // array of channelids which have asked to receive updates for this
+  // buffer.
+  kvec_t(uint64_t) update_channels;
+
+  int b_diff_failed;    // internal diff failed for this buffer
 };
 
 /*
@@ -816,6 +842,7 @@ struct tabpage_S {
   diff_T          *tp_first_diff;
   buf_T           *(tp_diffbuf[DB_COUNT]);
   int tp_diff_invalid;              ///< list of diffs is outdated
+  int tp_diff_update;               ///< update diffs before redrawing
   frame_T         *(tp_snapshot[SNAP_COUNT]);    ///< window layout snapshots
   ScopeDictDictItem tp_winvar;      ///< Variable for "t:" Dictionary.
   dict_T          *tp_vars;         ///< Internal variables, local to tab page.
@@ -892,9 +919,9 @@ typedef struct {
 /// Same as lpos_T, but with additional field len.
 typedef struct
 {
-    linenr_T    lnum;	///< line number
-    colnr_T     col;	///< column number
-    int         len;	///< length: 0 - to the end of line
+    linenr_T    lnum;   ///< line number
+    colnr_T     col;    ///< column number
+    int         len;    ///< length: 0 - to the end of line
 } llpos_T;
 
 /// posmatch_T provides an array for storing match items for matchaddpos()
@@ -902,10 +929,10 @@ typedef struct
 typedef struct posmatch posmatch_T;
 struct posmatch
 {
-    llpos_T     pos[MAXPOSMATCH];	///< array of positions
-    int         cur;			///< internal position counter 
-    linenr_T    toplnum;		///< top buffer line
-    linenr_T    botlnum;		///< bottom buffer line
+    llpos_T     pos[MAXPOSMATCH];   ///< array of positions
+    int         cur;                ///< internal position counter
+    linenr_T    toplnum;            ///< top buffer line
+    linenr_T    botlnum;            ///< bottom buffer line
 };
 
 /*
@@ -959,9 +986,11 @@ struct window_S {
                                        used to try to stay in the same column
                                        for up/down cursor motions. */
 
-  int w_set_curswant;               /* If set, then update w_curswant the next
-                                       time through cursupdate() to the
-                                       current virtual column */
+  int w_set_curswant;               // If set, then update w_curswant the next
+                                    // time through cursupdate() to the
+                                    // current virtual column
+
+  linenr_T w_last_cursorline;       ///< where last 'cursorline' was drawn
 
   // the next seven are used to update the visual part
   char w_old_visual_mode;           ///< last known VIsual_mode
@@ -1010,7 +1039,7 @@ struct window_S {
    * Recomputing is minimized by storing the result of computations.
    * Use functions in screen.c to check if they are valid and to update.
    * w_valid is a bitfield of flags, which indicate if specific values are
-   * valid or need to be recomputed.	
+   * valid or need to be recomputed.
    */
   int w_valid;
   pos_T w_valid_cursor;             /* last known position of w_cursor, used
@@ -1180,4 +1209,8 @@ static inline int win_hl_attr(win_T *wp, int hlf)
   return wp->w_hl_attrs[hlf];
 }
 
+/// Macros defined in Vim, but not in Neovim
+#define CHANGEDTICK(buf) \
+    (=== Include buffer.h & use buf_(get|set|inc)_changedtick ===)
+
 #endif // NVIM_BUFFER_DEFS_H
