@@ -1359,6 +1359,12 @@ static digr_T digraphdefault[] =
   { 'f', 't', 0xfb05 },
   { 's', 't', 0xfb06 },
 
+  // extra alternatives, easier to remember
+  { 'W', '`', 0x1e80 },
+  { 'w', '`', 0x1e81 },
+  { 'Y', '`', 0x1ef2 },
+  { 'y', '`', 0x1ef3 },
+
   // Vim 5.x compatible digraphs that don't conflict with the above
   { '~', '!', 161 },  // 
   { 'c', '|', 162 },  // 
@@ -1391,7 +1397,7 @@ static digr_T digraphdefault[] =
   { 'O', '`', 210 },  // 
   { 'O', '^', 212 },  // 
   { 'O', '~', 213 },  // 
-  { '/', '\\', 215 }, //  - multiplication symbol in ISO 8859-1
+  { '/', '\\', 215 },  //  - multiplication symbol in ISO 8859-1
   { 'U', '`', 217 },  // 
   { 'U', '^', 219 },  // 
   { 'I', 'p', 222 },  // 
@@ -1442,6 +1448,33 @@ int do_digraph(int c)
   return c;
 }
 
+/// Find a digraph for "val".  If found return the string to display it.
+/// If not found return NULL.
+char_u *get_digraph_for_char(int val)
+{
+  digr_T *dp;
+  static char_u r[3];
+
+  for (int use_defaults = 0; use_defaults <= 1; use_defaults++) {
+    if (use_defaults == 0) {
+      dp = (digr_T *)user_digraphs.ga_data;
+    } else {
+      dp = digraphdefault;
+    }
+    for (int i = 0;
+         use_defaults ? dp->char1 != NUL : i < user_digraphs.ga_len; i++) {
+      if (dp->result == val) {
+        r[0] = dp->char1;
+        r[1] = dp->char2;
+        r[2] = NUL;
+        return r;
+      }
+      dp++;
+    }
+  }
+  return NULL;
+}
+
 /// Get a digraph.  Used after typing CTRL-K on the command line or in normal
 /// mode.
 ///
@@ -1520,34 +1553,6 @@ static int getexactdigraph(int char1, int char2, int meta_char)
     }
   }
 
-  if ((retval != 0) && !enc_utf8) {
-    char_u buf[6], *to;
-    vimconv_T vc;
-
-    // Convert the Unicode digraph to 'encoding'.
-    int i = utf_char2bytes(retval, buf);
-    retval = 0;
-    vc.vc_type = CONV_NONE;
-
-    if (convert_setup(&vc, (char_u *)"utf-8", p_enc) == OK) {
-      vc.vc_fail = true;
-      assert(i >= 0);
-      size_t len = (size_t)i;
-      to = string_convert(&vc, buf, &len);
-
-      if (to != NULL) {
-        retval = (*mb_ptr2char)(to);
-        xfree(to);
-      }
-      (void)convert_setup(&vc, NULL, NULL);
-    }
-  }
-
-  // Ignore multi-byte characters when not in multi-byte mode.
-  if (!has_mbyte && (retval > 0xff)) {
-    retval = 0;
-  }
-
   if (retval == 0) {
     // digraph deleted or not found
     if ((char1 == ' ') && meta_char) {
@@ -1654,8 +1659,7 @@ void listdigraphs(void)
     tmp.result = getexactdigraph(tmp.char1, tmp.char2, FALSE);
 
     if ((tmp.result != 0)
-        && (tmp.result != tmp.char2)
-        && (has_mbyte || (tmp.result <= 255))) {
+        && (tmp.result != tmp.char2)) {
       printdigraph(&tmp);
     }
     dp++;
@@ -1668,9 +1672,6 @@ void listdigraphs(void)
     os_breakcheck();
     dp++;
   }
-  // clear screen, because some digraphs may be wrong, in which case we messed
-  // up ScreenLines
-  must_redraw = CLEAR;
 }
 
 static void printdigraph(digr_T *dp)
@@ -1696,17 +1697,23 @@ static void printdigraph(digr_T *dp)
       }
     }
 
-    p = buf;
+    p = &buf[0];
     *p++ = dp->char1;
     *p++ = dp->char2;
     *p++ = ' ';
+    *p = NUL;
+    msg_outtrans(buf);
+    p = buf;
 
     // add a space to draw a composing char on
     if (utf_iscomposing(dp->result)) {
       *p++ = ' ';
     }
-    p += (*mb_char2bytes)(dp->result, p);
+    p += utf_char2bytes(dp->result, p);
 
+    *p = NUL;
+    msg_outtrans_attr(buf, HL_ATTR(HLF_8));
+    p = buf;
     if (char2cells(dp->result) == 1) {
       *p++ = ' ';
     }
@@ -1827,12 +1834,12 @@ void ex_loadkeymap(exarg_T *eap)
     xfree(line);
   }
 
-  // setup ":lnoremap" to map the keys
-  for (int i = 0; i < curbuf->b_kmap_ga.ga_len; ++i) {
+  // setup ":lmap" to map the keys
+  for (int i = 0; i < curbuf->b_kmap_ga.ga_len; i++) {
     vim_snprintf((char *)buf, sizeof(buf), "<buffer> %s %s",
                  ((kmap_T *)curbuf->b_kmap_ga.ga_data)[i].from,
                  ((kmap_T *)curbuf->b_kmap_ga.ga_data)[i].to);
-    (void)do_map(2, buf, LANGMAP, FALSE);
+    (void)do_map(0, buf, LANGMAP, false);
   }
 
   p_cpo = save_cpo;
@@ -1841,6 +1848,16 @@ void ex_loadkeymap(exarg_T *eap)
   status_redraw_curbuf();
 }
 
+/// Frees the buf_T.b_kmap_ga field of a buffer.
+void keymap_ga_clear(garray_T *kmap_ga)
+{
+  kmap_T *kp = (kmap_T *)kmap_ga->ga_data;
+  for (int i = 0; i < kmap_ga->ga_len; i++) {
+    xfree(kp[i].from);
+    xfree(kp[i].to);
+  }
+}
+
 /// Stop using 'keymap'.
 static void keymap_unload(void)
 {
@@ -1858,12 +1875,11 @@ static void keymap_unload(void)
   // clear the ":lmap"s
   kp = (kmap_T *)curbuf->b_kmap_ga.ga_data;
 
-  for (int i = 0; i < curbuf->b_kmap_ga.ga_len; ++i) {
+  for (int i = 0; i < curbuf->b_kmap_ga.ga_len; i++) {
     vim_snprintf((char *)buf, sizeof(buf), "<buffer> %s", kp[i].from);
-    (void)do_map(1, buf, LANGMAP, FALSE);
-    xfree(kp[i].from);
-    xfree(kp[i].to);
+    (void)do_map(1, buf, LANGMAP, false);
   }
+  keymap_ga_clear(&curbuf->b_kmap_ga);
 
   p_cpo = save_cpo;
 
