@@ -19,17 +19,17 @@ struct process {
   Loop *loop;
   void *data;
   int pid, status, refcount;
-  // set to the hrtime of when process_stop was called for the process.
-  uint64_t stopped_time;
+  uint64_t stopped_time;  // process_stop() timestamp
   const char *cwd;
   char **argv;
-  Stream *in, *out, *err;
+  Stream in, out, err;
   process_exit_cb cb;
   internal_process_cb internal_exit_cb, internal_close_cb;
   bool closed, detach;
   MultiQueue *events;
 };
 
+
 static inline Process process_init(Loop *loop, ProcessType type, void *data)
 {
   return (Process) {
@@ -38,14 +38,14 @@ static inline Process process_init(Loop *loop, ProcessType type, void *data)
     .loop = loop,
     .events = NULL,
     .pid = 0,
-    .status = 0,
+    .status = -1,
     .refcount = 0,
     .stopped_time = 0,
     .cwd = NULL,
     .argv = NULL,
-    .in = NULL,
-    .out = NULL,
-    .err = NULL,
+    .in = { .closed = false },
+    .out = { .closed = false },
+    .err = { .closed = false },
     .cb = NULL,
     .closed = false,
     .internal_close_cb = NULL,
@@ -54,6 +54,11 @@ static inline Process process_init(Loop *loop, ProcessType type, void *data)
   };
 }
 
+static inline bool process_is_stopped(Process *proc)
+{
+  return proc->stopped_time != 0;
+}
+
 #ifdef INCLUDE_GENERATED_DECLARATIONS
 # include "event/process.h.generated.h"
 #endif
