@@ -10,11 +10,13 @@
 
 #include "nvim/vim.h"
 #include "nvim/eval.h"
+#include "nvim/highlight.h"
 #include "nvim/memfile.h"
 #include "nvim/memory.h"
 #include "nvim/message.h"
 #include "nvim/misc1.h"
 #include "nvim/ui.h"
+#include "nvim/api/vim.h"
 
 #ifdef HAVE_JEMALLOC
 // Force je_ prefix on jemalloc functions.
@@ -66,7 +68,7 @@ void try_to_free_memory(void)
   trying_to_free = true;
 
   // free any scrollback text
-  clear_sb_text();
+  clear_sb_text(true);
   // Try to save all buffers and release as many blocks as possible
   mf_release_all();
 
@@ -369,10 +371,12 @@ char *xstpncpy(char *restrict dst, const char *restrict src, size_t maxlen)
 /// string that fits in the buffer (unless, of course, the buffer size is
 /// zero). It does not pad out the result like strncpy() does.
 ///
-/// @param dst    Buffer to store the result
-/// @param src    String to be copied
-/// @param dsize  Size of `dst`
-/// @return       strlen(src). If retval >= dstsize, truncation occurs.
+/// @param[out]  dst  Buffer to store the result.
+/// @param[in]  src  String to be copied.
+/// @param[in]  dsize  Size of `dst`.
+///
+/// @return Length of `src`. May be greater than `dsize - 1`, which would mean
+///         that string was truncated.
 size_t xstrlcpy(char *restrict dst, const char *restrict src, size_t dsize)
   FUNC_ATTR_NONNULL_ALL
 {
@@ -394,11 +398,13 @@ size_t xstrlcpy(char *restrict dst, const char *restrict src, size_t dsize)
 /// @see vim_strcat from Vim.
 /// @see strlcat from OpenBSD.
 ///
-/// @param dst      Buffer to be appended-to. Must have a NUL byte.
-/// @param src      String to put at the end of `dst`
-/// @param dsize    Size of `dst` including NUL byte. Must be greater than 0.
-/// @return         strlen(src) + strlen(initial dst)
-///                 If retval >= dsize, truncation occurs.
+/// @param[in,out]  dst  Buffer to be appended-to. Must have a NUL byte.
+/// @param[in]  src  String to put at the end of `dst`.
+/// @param[in]  dsize  Size of `dst` including NUL byte. Must be greater than 0.
+///
+/// @return Length of the resulting string as if destination size was #SIZE_MAX.
+///         May be greater than `dsize - 1`, which would mean that string was
+///         truncated.
 size_t xstrlcat(char *const dst, const char *const src, const size_t dsize)
   FUNC_ATTR_NONNULL_ALL
 {
@@ -559,6 +565,7 @@ void time_to_bytes(time_t time_, uint8_t buf[8])
 #include "nvim/tag.h"
 #include "nvim/window.h"
 #include "nvim/os/os.h"
+#include "nvim/eval/typval.h"
 
 /*
  * Free everything that we allocated.
@@ -613,7 +620,6 @@ void free_all_mem(void)
 
   /* Obviously named calls. */
   free_all_autocmds();
-  free_all_options();
   free_all_marks();
   alist_clear(&global_alist);
   free_homedir();
@@ -628,7 +634,7 @@ void free_all_mem(void)
   free_signs();
   set_expr_line(NULL);
   diff_clear(curtab);
-  clear_sb_text();            /* free any scrollback text */
+  clear_sb_text(true);            // free any scrollback text
 
   /* Free some global vars. */
   xfree(last_cmdline);
@@ -651,6 +657,9 @@ void free_all_mem(void)
   /* Destroy all windows.  Must come before freeing buffers. */
   win_free_all();
 
+  // Free all option values.  Must come after closing windows.
+  free_all_options();
+
   free_cmdline_buf();
 
   /* Clear registers. */
@@ -673,6 +682,7 @@ void free_all_mem(void)
       break;
 
   eval_clear();
+  api_vim_free_all_mem();
 
   // Free all buffers.  Reset 'autochdir' to avoid accessing things that
   // were freed already.
@@ -691,7 +701,8 @@ void free_all_mem(void)
   /* screenlines (can't display anything now!) */
   free_screenlines();
 
-  clear_hl_tables();
+  clear_hl_tables(false);
+  list_free_log();
 }
 
 #endif
