@@ -286,7 +286,7 @@ void trunc_string(char_u *s, char_u *buf, int room_in, int buflen)
   half = i = (int)STRLEN(s);
   for (;;) {
     do {
-      half = half - (*mb_head_off)(s, s + half - 1) - 1;
+      half = half - utf_head_off(s, s + half - 1) - 1;
     } while (half > 0 && utf_iscomposing(utf_ptr2char(s + half)));
     n = ptr2cells(s + half);
     if (len + n > room || half == 0) {
@@ -331,6 +331,7 @@ void trunc_string(char_u *s, char_u *buf, int room_in, int buflen)
  */
 
 int smsg(char *s, ...)
+  FUNC_ATTR_PRINTF(1, 2)
 {
   va_list arglist;
 
@@ -341,6 +342,7 @@ int smsg(char *s, ...)
 }
 
 int smsg_attr(int attr, char *s, ...)
+  FUNC_ATTR_PRINTF(2, 3)
 {
   va_list arglist;
 
@@ -434,7 +436,7 @@ void msg_source(int attr)
   }
   p = get_emsg_lnum();
   if (p != NULL) {
-    msg_attr(p, hl_attr(HLF_N));
+    msg_attr(p, HL_ATTR(HLF_N));
     xfree(p);
     last_sourcing_lnum = sourcing_lnum;      /* only once for each line */
   }
@@ -487,9 +489,6 @@ int emsg(const char_u *s_)
   }
 
   called_emsg = true;
-  if (emsg_silent == 0) {
-    ex_exitval = 1;
-  }
 
   // If "emsg_severe" is TRUE: When an error exception is to be thrown,
   // prefer this message over previous messages for the same command.
@@ -506,7 +505,7 @@ int emsg(const char_u *s_)
      */
     if (cause_errthrow((char_u *)s, severe, &ignore) == true) {
       if (!ignore) {
-        did_emsg = true;
+        did_emsg++;
       }
       return true;
     }
@@ -540,9 +539,11 @@ int emsg(const char_u *s_)
       return true;
     }
 
+    ex_exitval = 1;
+
     // Reset msg_silent, an error causes messages to be switched back on.
     msg_silent = 0;
-    cmd_silent = FALSE;
+    cmd_silent = false;
 
     if (global_busy) {        // break :global command
       global_busy++;
@@ -551,14 +552,14 @@ int emsg(const char_u *s_)
     if (p_eb) {
       beep_flush();           // also includes flush_buffers()
     } else {
-      flush_buffers(false);   // flush internal buffers
+      flush_buffers(FLUSH_MINIMAL);  // flush internal buffers
     }
-    did_emsg = true;          // flag for DoOneCmd()
+    did_emsg++;               // flag for DoOneCmd()
   }
 
   emsg_on_display = true;     // remember there is an error message
   msg_scroll++;               // don't overwrite a previous message
-  attr = hl_attr(HLF_E);      // set highlight mode for error messages
+  attr = HL_ATTR(HLF_E);      // set highlight mode for error messages
   if (msg_scrolled != 0) {
     need_wait_return = true;  // needed in case emsg() is called after
   }                           // wait_return has reset need_wait_return
@@ -582,20 +583,62 @@ void emsg_invreg(int name)
 
 /// Print an error message with unknown number of arguments
 bool emsgf(const char *const fmt, ...)
+  FUNC_ATTR_PRINTF(1, 2)
+{
+  bool ret;
+
+  va_list ap;
+  va_start(ap, fmt);
+  ret = emsgfv(fmt, ap);
+  va_end(ap);
+
+  return ret;
+}
+
+/// Print an error message with unknown number of arguments
+static bool emsgfv(const char *fmt, va_list ap)
 {
   static char errbuf[IOSIZE];
   if (emsg_not_now()) {
     return true;
   }
 
-  va_list ap;
-  va_start(ap, fmt);
   vim_vsnprintf(errbuf, sizeof(errbuf), fmt, ap, NULL);
-  va_end(ap);
 
   return emsg((const char_u *)errbuf);
 }
 
+/// Same as emsg(...), but abort on error when ABORT_ON_INTERNAL_ERROR is
+/// defined. It is used for internal errors only, so that they can be
+/// detected when fuzzing vim.
+void iemsg(const char *s)
+{
+    emsg((char_u *)s);
+#ifdef ABORT_ON_INTERNAL_ERROR
+    abort();
+#endif
+}
+
+/// Same as emsgf(...) but abort on error when ABORT_ON_INTERNAL_ERROR is
+/// defined. It is used for internal errors only, so that they can be
+/// detected when fuzzing vim.
+void iemsgf(const char *s, ...)
+{
+    va_list ap;
+    va_start(ap, s);
+    (void)emsgfv(s, ap);
+    va_end(ap);
+#ifdef ABORT_ON_INTERNAL_ERROR
+    abort();
+#endif
+}
+
+/// Give an "Internal error" message.
+void internal_error(char *where)
+{
+    IEMSG2(_(e_intern2), where);
+}
+
 static void msg_emsgf_event(void **argv)
 {
   char *s = argv[0];
@@ -604,6 +647,7 @@ static void msg_emsgf_event(void **argv)
 }
 
 void msg_schedule_emsgf(const char *const fmt, ...)
+  FUNC_ATTR_PRINTF(1, 2)
 {
   va_list ap;
   va_start(ap, fmt);
@@ -659,8 +703,8 @@ char_u *msg_may_trunc(int force, char_u *s)
         return s;
 
       for (n = 0; size >= room; ) {
-        size -= (*mb_ptr2cells)(s + n);
-        n += (*mb_ptr2len)(s + n);
+        size -= utf_ptr2cells(s + n);
+        n += utfc_ptr2len(s + n);
       }
       --n;
     }
@@ -788,12 +832,11 @@ void msg_end_prompt(void)
   lines_left = -1;
 }
 
-/*
- * wait for the user to hit a key (normally a return)
- * if 'redraw' is TRUE, clear and redraw the screen
- * if 'redraw' is FALSE, just redraw the screen
- * if 'redraw' is -1, don't redraw at all
- */
+/// wait for the user to hit a key (normally a return)
+///
+/// if 'redraw' is true, redraw the entire screen NOT_VALID
+/// if 'redraw' is false, do a normal redraw
+/// if 'redraw' is -1, don't redraw at all
 void wait_return(int redraw)
 {
   int c;
@@ -803,8 +846,9 @@ void wait_return(int redraw)
   int save_Recording;
   FILE        *save_scriptout;
 
-  if (redraw == TRUE)
-    must_redraw = CLEAR;
+  if (redraw == true) {
+    redraw_all_later(NOT_VALID);
+  }
 
   /* If using ":silent cmd", don't wait for a return.  Also don't set
    * need_wait_return to do it later. */
@@ -994,9 +1038,10 @@ static void hit_return_msg(void)
   if (got_int)
     MSG_PUTS(_("Interrupt: "));
 
-  MSG_PUTS_ATTR(_("Press ENTER or type command to continue"), hl_attr(HLF_R));
-  if (!msg_use_printf())
+  MSG_PUTS_ATTR(_("Press ENTER or type command to continue"), HL_ATTR(HLF_R));
+  if (!msg_use_printf()) {
     msg_clr_eos();
+  }
   p_more = save_p_more;
 }
 
@@ -1080,7 +1125,7 @@ void msg_putchar_attr(int c, int attr)
     buf[2] = (char)K_THIRD(c);
     buf[3] = NUL;
   } else {
-    buf[(*mb_char2bytes)(c, (char_u *)buf)] = NUL;
+    buf[utf_char2bytes(c, (char_u *)buf)] = NUL;
   }
   msg_puts_attr(buf, attr);
 }
@@ -1100,7 +1145,7 @@ void msg_home_replace(char_u *fname)
 
 void msg_home_replace_hl(char_u *fname)
 {
-  msg_home_replace_attr(fname, hl_attr(HLF_D));
+  msg_home_replace_attr(fname, HL_ATTR(HLF_D));
 }
 
 static void msg_home_replace_attr(char_u *fname, int attr)
@@ -1178,10 +1223,10 @@ int msg_outtrans_len_attr(char_u *msgstr, int len, int attr)
     // Don't include composing chars after the end.
     mb_l = utfc_ptr2len_len((char_u *)str, len + 1);
     if (mb_l > 1) {
-      c = (*mb_ptr2char)((char_u *)str);
+      c = utf_ptr2char((char_u *)str);
       if (vim_isprintc(c)) {
         // Printable multi-byte char: count the cells.
-        retval += (*mb_ptr2cells)((char_u *)str);
+        retval += utf_ptr2cells((char_u *)str);
       } else {
         // Unprintable multi-byte char: print the printable chars so
         // far and the translation of the unprintable char.
@@ -1190,7 +1235,7 @@ int msg_outtrans_len_attr(char_u *msgstr, int len, int attr)
         }
         plain_start = str + mb_l;
         msg_puts_attr((const char *)transchar(c),
-                      (attr == 0 ? hl_attr(HLF_8) : attr));
+                      (attr == 0 ? HL_ATTR(HLF_8) : attr));
         retval += char2cells(c);
       }
       len -= mb_l - 1;
@@ -1204,7 +1249,7 @@ int msg_outtrans_len_attr(char_u *msgstr, int len, int attr)
           msg_puts_attr_len(plain_start, str - plain_start, attr);
         }
         plain_start = str + 1;
-        msg_puts_attr((const char *)s, attr == 0 ? hl_attr(HLF_8) : attr);
+        msg_puts_attr((const char *)s, attr == 0 ? HL_ATTR(HLF_8) : attr);
         retval += (int)STRLEN(s);
       } else {
         retval++;
@@ -1237,31 +1282,30 @@ void msg_make(char_u *arg)
   }
 }
 
-/*
- * Output the string 'str' upto a NUL character.
- * Return the number of characters it takes on the screen.
- *
- * If K_SPECIAL is encountered, then it is taken in conjunction with the
- * following character and shown as <F1>, <S-Up> etc.  Any other character
- * which is not printable shown in <> form.
- * If 'from' is TRUE (lhs of a mapping), a space is shown as <Space>.
- * If a character is displayed in one of these special ways, is also
- * highlighted (its highlight name is '8' in the p_hl variable).
- * Otherwise characters are not highlighted.
- * This function is used to show mappings, where we want to see how to type
- * the character/string -- webb
- */
-int
-msg_outtrans_special (
-    char_u *strstart,
-    int from               /* TRUE for lhs of a mapping */
+/// Output the string 'str' upto a NUL character.
+/// Return the number of characters it takes on the screen.
+///
+/// If K_SPECIAL is encountered, then it is taken in conjunction with the
+/// following character and shown as <F1>, <S-Up> etc.  Any other character
+/// which is not printable shown in <> form.
+/// If 'from' is TRUE (lhs of a mapping), a space is shown as <Space>.
+/// If a character is displayed in one of these special ways, is also
+/// highlighted (its highlight name is '8' in the p_hl variable).
+/// Otherwise characters are not highlighted.
+/// This function is used to show mappings, where we want to see how to type
+/// the character/string -- webb
+int msg_outtrans_special(
+    const char_u *strstart,
+    int from               ///< true for LHS of a mapping
 )
 {
-  char_u      *str = strstart;
+  if (strstart == NULL) {
+    return 0;  // Do nothing.
+  }
+  const char_u *str = strstart;
   int retval = 0;
-  int attr;
+  int attr = HL_ATTR(HLF_8);
 
-  attr = hl_attr(HLF_8);
   while (*str != NUL) {
     const char *string;
     // Leading and trailing spaces need to be displayed in <> form.
@@ -1307,7 +1351,7 @@ char *str2special_save(const char *const str, const bool replace_spaces,
   return (char *)ga.ga_data;
 }
 
-/// Convert character, replacing key one key code with printable representation
+/// Convert character, replacing key with printable representation.
 ///
 /// @param[in,out]  sp  String to convert. Is advanced to the next key code.
 /// @param[in]  replace_spaces  Convert spaces into <Space>, normally used for
@@ -1343,9 +1387,6 @@ const char *str2special(const char **const sp, const bool replace_spaces,
     if (c == K_SPECIAL && str[1] != NUL && str[2] != NUL) {
       c = TO_SPECIAL((uint8_t)str[1], (uint8_t)str[2]);
       str += 2;
-      if (c == KS_ZERO) {  // display <Nul> as ^@ or <Nul>
-        c = NUL;
-      }
     }
     if (IS_SPECIAL(c) || modifiers) {  // Special key.
       special = true;
@@ -1376,7 +1417,7 @@ const char *str2special(const char **const sp, const bool replace_spaces,
       || (replace_lt && c == '<')) {
     return (const char *)get_special_key_name(c, modifiers);
   }
-  buf[0] = c;
+  buf[0] = (char)c;
   buf[1] = NUL;
   return buf;
 }
@@ -1392,7 +1433,7 @@ void str2specialbuf(const char *sp, char *buf, size_t len)
   while (*sp) {
     const char *s = str2special(&sp, false, false);
     const size_t s_len = strlen(s);
-    if (s_len <= len) {
+    if (len <= s_len) {
       break;
     }
     memcpy(buf, s, s_len);
@@ -1434,18 +1475,20 @@ void msg_prt_line(char_u *s, int list)
 
   while (!got_int) {
     if (n_extra > 0) {
-      --n_extra;
-      if (c_extra)
+      n_extra--;
+      if (c_extra) {
         c = c_extra;
-      else
+      } else {
+        assert(p_extra != NULL);
         c = *p_extra++;
-    } else if (has_mbyte && (l = (*mb_ptr2len)(s)) > 1) {
-      col += (*mb_ptr2cells)(s);
+      }
+    } else if ((l = utfc_ptr2len(s)) > 1) {
+      col += utf_ptr2cells(s);
       char buf[MB_MAXBYTES + 1];
       if (lcs_nbsp != NUL && list
-          && (mb_ptr2char(s) == 160 || mb_ptr2char(s) == 0x202f)) {
-        mb_char2bytes(lcs_nbsp, (char_u *)buf);
-        buf[(*mb_ptr2len)((char_u *)buf)] = NUL;
+          && (utf_ptr2char(s) == 160 || utf_ptr2char(s) == 0x202f)) {
+        utf_char2bytes(lcs_nbsp, (char_u *)buf);
+        buf[utfc_ptr2len((char_u *)buf)] = NUL;
       } else {
         memmove(buf, s, (size_t)l);
         buf[l] = NUL;
@@ -1465,18 +1508,18 @@ void msg_prt_line(char_u *s, int list)
         } else {
           c = lcs_tab1;
           c_extra = lcs_tab2;
-          attr = hl_attr(HLF_8);
+          attr = HL_ATTR(HLF_8);
         }
       } else if (c == 160 && list && lcs_nbsp != NUL) {
         c = lcs_nbsp;
-        attr = hl_attr(HLF_8);
+        attr = HL_ATTR(HLF_8);
       } else if (c == NUL && list && lcs_eol != NUL) {
         p_extra = (char_u *)"";
         c_extra = NUL;
         n_extra = 1;
         c = lcs_eol;
-        attr = hl_attr(HLF_AT);
-        --s;
+        attr = HL_ATTR(HLF_AT);
+        s--;
       } else if (c != NUL && (n = byte2cells(c)) > 1) {
         n_extra = n - 1;
         p_extra = transchar_byte(c);
@@ -1484,13 +1527,13 @@ void msg_prt_line(char_u *s, int list)
         c = *p_extra++;
         /* Use special coloring to be able to distinguish <hex> from
          * the same in plain text. */
-        attr = hl_attr(HLF_8);
+        attr = HL_ATTR(HLF_8);
       } else if (c == ' ' && trail != NULL && s > trail) {
         c = lcs_trail;
-        attr = hl_attr(HLF_8);
+        attr = HL_ATTR(HLF_8);
       } else if (c == ' ' && list && lcs_space != NUL) {
         c = lcs_space;
-        attr = hl_attr(HLF_8);
+        attr = HL_ATTR(HLF_8);
       }
     }
 
@@ -1511,13 +1554,12 @@ static char_u *screen_puts_mbyte(char_u *s, int l, int attr)
 {
   int cw;
 
-  msg_didout = TRUE;            /* remember that line is not empty */
-  cw = (*mb_ptr2cells)(s);
-  if (cw > 1 && (
-        cmdmsg_rl ? msg_col <= 1 :
-        msg_col == Columns - 1)) {
-    /* Doesn't fit, print a highlighted '>' to fill it up. */
-    msg_screen_putchar('>', hl_attr(HLF_AT));
+  msg_didout = true;            // remember that line is not empty
+  cw = utf_ptr2cells(s);
+  if (cw > 1
+      && (cmdmsg_rl ? msg_col <= 1 : msg_col == Columns - 1)) {
+    // Doesn't fit, print a highlighted '>' to fill it up.
+    msg_screen_putchar('>', HL_ATTR(HLF_AT));
     return s;
   }
 
@@ -1549,7 +1591,7 @@ void msg_puts(const char *s)
 
 void msg_puts_title(const char *s)
 {
-  msg_puts_attr(s, hl_attr(HLF_T));
+  msg_puts_attr(s, HL_ATTR(HLF_T));
 }
 
 /*
@@ -1571,7 +1613,7 @@ void msg_puts_long_len_attr(char_u *longstr, int len, int attr)
   if (len > room && room >= 20) {
     slen = (room - 3) / 2;
     msg_outtrans_len_attr(longstr, slen, attr);
-    msg_puts_attr("...", hl_attr(HLF_8));
+    msg_puts_attr("...", HL_ATTR(HLF_8));
   }
   msg_outtrans_len_attr(longstr + len - slen, slen, attr);
 }
@@ -1636,7 +1678,7 @@ void msg_puts_attr_len(const char *const str, const ptrdiff_t len, int attr)
 /// @param[in]  attr  Highlight attributes.
 /// @param[in]  fmt  Format string.
 void msg_printf_attr(const int attr, const char *const fmt, ...)
-  FUNC_ATTR_NONNULL_ARG(2)
+  FUNC_ATTR_NONNULL_ARG(2) FUNC_ATTR_PRINTF(2, 3)
 {
   static char msgbuf[IOSIZE];
 
@@ -1675,14 +1717,12 @@ static void msg_puts_display(const char_u *str, int maxlen, int attr,
         && (*s == '\n' || (cmdmsg_rl
                            ? (msg_col <= 1
                               || (*s == TAB && msg_col <= 7)
-                              || (has_mbyte
-                                  && (*mb_ptr2cells)(s) > 1
+                              || (utf_ptr2cells(s) > 1
                                   && msg_col <= 2))
                            : (msg_col + t_col >= Columns - 1
                               || (*s == TAB
                                   && msg_col + t_col >= ((Columns - 1) & ~7))
-                              || (has_mbyte
-                                  && (*mb_ptr2cells)(s) > 1
+                              || (utf_ptr2cells(s) > 1
                                   && msg_col + t_col >= Columns - 2))))) {
       // The screen is scrolled up when at the last row (some terminals
       // scroll automatically, some don't.  To avoid problems we scroll
@@ -1723,12 +1763,11 @@ static void msg_puts_display(const char_u *str, int maxlen, int attr,
       }
 
       inc_msg_scrolled();
-      need_wait_return = TRUE;       /* may need wait_return in main() */
-      if (must_redraw < VALID)
-        must_redraw = VALID;
-      redraw_cmdline = TRUE;
-      if (cmdline_row > 0 && !exmode_active)
-        --cmdline_row;
+      need_wait_return = true;       // may need wait_return in main()
+      redraw_cmdline = true;
+      if (cmdline_row > 0 && !exmode_active) {
+        cmdline_row--;
+      }
 
       /*
        * If screen is completely filled and 'more' is set then wait
@@ -1752,7 +1791,7 @@ static void msg_puts_display(const char_u *str, int maxlen, int attr,
 
     wrap = *s == '\n'
            || msg_col + t_col >= Columns
-           || (has_mbyte && (*mb_ptr2cells)(s) > 1
+           || (utf_ptr2cells(s) > 1
                && msg_col + t_col >= Columns - 1)
     ;
     if (t_col > 0 && (wrap || *s == '\r' || *s == '\b'
@@ -1785,17 +1824,13 @@ static void msg_puts_display(const char_u *str, int maxlen, int attr,
       } while (msg_col & 7);
     } else if (*s == BELL) {  // beep (from ":sh")
       vim_beep(BO_SH);
-    } else {
-      if (has_mbyte) {
-        cw = (*mb_ptr2cells)(s);
-        if (enc_utf8 && maxlen >= 0)
-          /* avoid including composing chars after the end */
-          l = utfc_ptr2len_len(s, (int)((str + maxlen) - s));
-        else
-          l = (*mb_ptr2len)(s);
+    } else if (*s >= 0x20) {  // printable char
+      cw = utf_ptr2cells(s);
+      if (maxlen >= 0) {
+        // avoid including composing chars after the end
+        l = utfc_ptr2len_len(s, (int)((str + maxlen) - s));
       } else {
-        cw = 1;
-        l = 1;
+        l = utfc_ptr2len(s);
       }
       // When drawing from right to left or when a double-wide character
       // doesn't fit, draw a single character here.  Otherwise collect
@@ -1840,13 +1875,30 @@ bool message_filtered(char_u *msg)
   return cmdmod.filter_force ? match : !match;
 }
 
+/// including horizontal separator
+int msg_scrollsize(void)
+{
+  return msg_scrolled + p_ch + 1;
+}
+
 /*
  * Scroll the screen up one line for displaying the next message line.
  */
 static void msg_scroll_up(void)
 {
-  /* scrolling up always works */
-  screen_del_lines(0, 0, 1, (int)Rows, NULL);
+  if (dy_flags & DY_MSGSEP) {
+    if (msg_scrolled == 0) {
+      screen_fill(Rows-p_ch-1, Rows-p_ch, 0, (int)Columns,
+                  fill_msgsep, fill_msgsep, HL_ATTR(HLF_MSGSEP));
+    }
+    int nscroll = MIN(msg_scrollsize()+1, Rows);
+    screen_del_lines(Rows-nscroll, 1, Rows, 0, Columns);
+  } else {
+    screen_del_lines(0, 1, (int)Rows, 0, Columns);
+  }
+  // TODO(bfredl): when msgsep display is properly batched, this fill should be
+  // eliminated.
+  screen_fill(Rows-1, Rows, 0, (int)Columns, ' ', ' ', 0);
 }
 
 /*
@@ -1873,30 +1925,38 @@ static void inc_msg_scrolled(void)
     xfree(tofree);
   }
   msg_scrolled++;
+  if (must_redraw < VALID) {
+    must_redraw = VALID;
+  }
 }
 
-static msgchunk_T *last_msgchunk = NULL; /* last displayed text */
+static msgchunk_T *last_msgchunk = NULL;  // last displayed text
 
+typedef enum {
+  SB_CLEAR_NONE = 0,
+  SB_CLEAR_ALL,
+  SB_CLEAR_CMDLINE_BUSY,
+  SB_CLEAR_CMDLINE_DONE
+} sb_clear_T;
 
-static int do_clear_sb_text = FALSE;    /* clear text on next msg */
+// When to clear text on next msg.
+static sb_clear_T do_clear_sb_text = SB_CLEAR_NONE;
 
-/*
- * Store part of a printed message for displaying when scrolling back.
- */
-static void
-store_sb_text (
-    char_u **sb_str,           /* start of string */
-    char_u *s,                 /* just after string */
+/// Store part of a printed message for displaying when scrolling back.
+static void store_sb_text(
+    char_u **sb_str,           // start of string
+    char_u *s,                 // just after string
     int attr,
     int *sb_col,
-    int finish                     /* line ends */
+    int finish                     // line ends
 )
 {
   msgchunk_T  *mp;
 
-  if (do_clear_sb_text) {
-    clear_sb_text();
-    do_clear_sb_text = FALSE;
+  if (do_clear_sb_text == SB_CLEAR_ALL
+      || do_clear_sb_text == SB_CLEAR_CMDLINE_DONE) {
+    clear_sb_text(do_clear_sb_text == SB_CLEAR_ALL);
+    do_clear_sb_text = SB_CLEAR_NONE;
   }
 
   if (s > *sb_str) {
@@ -1928,21 +1988,43 @@ store_sb_text (
  */
 void may_clear_sb_text(void)
 {
-  do_clear_sb_text = TRUE;
+  do_clear_sb_text = SB_CLEAR_ALL;
 }
 
-/*
- * Clear any text remembered for scrolling back.
- * Called when redrawing the screen.
- */
-void clear_sb_text(void)
+/// Starting to edit the command line, do not clear messages now.
+void sb_text_start_cmdline(void)
+{
+  do_clear_sb_text = SB_CLEAR_CMDLINE_BUSY;
+  msg_sb_eol();
+}
+
+/// Ending to edit the command line.  Clear old lines but the last one later.
+void sb_text_end_cmdline(void)
+{
+  do_clear_sb_text = SB_CLEAR_CMDLINE_DONE;
+}
+
+/// Clear any text remembered for scrolling back.
+/// When "all" is FALSE keep the last line.
+/// Called when redrawing the screen.
+void clear_sb_text(int all)
 {
   msgchunk_T  *mp;
+  msgchunk_T  **lastp;
 
-  while (last_msgchunk != NULL) {
-    mp = last_msgchunk->sb_prev;
-    xfree(last_msgchunk);
-    last_msgchunk = mp;
+  if (all) {
+    lastp = &last_msgchunk;
+  } else {
+    if (last_msgchunk == NULL) {
+      return;
+    }
+    lastp = &last_msgchunk->sb_prev;
+  }
+
+  while (*lastp != NULL) {
+    mp = (*lastp)->sb_prev;
+    xfree(*lastp);
+    *lastp = mp;
   }
 }
 
@@ -2028,8 +2110,9 @@ static void t_puts(int *t_col, const char_u *t_s, const char_u *s, int attr)
   }
 }
 
-// Returns TRUE when messages should be printed to stdout/stderr, which
-// happens when no UIs are attached and nvim is not being embedded
+// Returns TRUE when messages should be printed to stdout/stderr:
+//    - "batch mode" ("silent mode", -es/-Es)
+//    - no UI and not embedded
 int msg_use_printf(void)
 {
   return !embedded_mode && !ui_active();
@@ -2181,8 +2264,8 @@ static int do_more_prompt(int typed_char)
         skip_redraw = TRUE;                     /* skip redraw once */
         need_wait_return = FALSE;               /* don't wait in main() */
       }
-    /*FALLTHROUGH*/
-    case 'q':                   /* quit */
+      FALLTHROUGH;
+    case 'q':                   // quit
     case Ctrl_C:
     case ESC:
       if (confirm_msg_used) {
@@ -2229,9 +2312,10 @@ static int do_more_prompt(int typed_char)
               mp_last = msg_sb_start(mp_last->sb_prev);
           }
 
-          if (toscroll == -1 && screen_ins_lines(0, 0, 1,
-                  (int)Rows, NULL) == OK) {
-            /* display line at top */
+          if (toscroll == -1
+              && screen_ins_lines(0, 1, (int)Rows, 0, (int)Columns) == OK) {
+            screen_fill(0, 1, 0, (int)Columns, ' ', ' ', 0);
+            // display line at top
             (void)disp_sb_line(0, mp);
           } else {
             /* redisplay all lines */
@@ -2300,10 +2384,9 @@ static int do_more_prompt(int typed_char)
  * yet.  When stderr can't be used, collect error messages until the GUI has
  * started and they can be displayed in a message box.
  */
-void mch_errmsg(char *str)
+void mch_errmsg(const char *const str)
+  FUNC_ATTR_NONNULL_ALL
 {
-  int len;
-
 #ifdef UNIX
   /* On Unix use stderr if it's a tty.
    * When not going to start the GUI also use stderr.
@@ -2317,14 +2400,13 @@ void mch_errmsg(char *str)
   /* avoid a delay for a message that isn't there */
   emsg_on_display = FALSE;
 
-  len = (int)STRLEN(str) + 1;
+  const size_t len = strlen(str) + 1;
   if (error_ga.ga_data == NULL) {
     ga_set_growsize(&error_ga, 80);
     error_ga.ga_itemsize = 1;
   }
   ga_grow(&error_ga, len);
-  memmove((char_u *)error_ga.ga_data + error_ga.ga_len,
-      (char_u *)str, len);
+  memmove(error_ga.ga_data + error_ga.ga_len, str, len);
 #ifdef UNIX
   /* remove CR characters, they are displayed */
   {
@@ -2390,7 +2472,7 @@ void msg_moremsg(int full)
   int attr;
   char_u      *s = (char_u *)_("-- More --");
 
-  attr = hl_attr(HLF_M);
+  attr = HL_ATTR(HLF_M);
   screen_puts(s, (int)Rows - 1, 0, attr);
   if (full)
     screen_puts((char_u *)
@@ -2673,7 +2755,7 @@ void give_warning(char_u *message, bool hl) FUNC_ATTR_NONNULL_ARG(1)
   xfree(keep_msg);
   keep_msg = NULL;
   if (hl) {
-    keep_msg_attr = hl_attr(HLF_W);
+    keep_msg_attr = HL_ATTR(HLF_W);
   } else {
     keep_msg_attr = 0;
   }
@@ -2687,6 +2769,12 @@ void give_warning(char_u *message, bool hl) FUNC_ATTR_NONNULL_ARG(1)
   --no_wait_return;
 }
 
+void give_warning2(char_u *const message, char_u *const a1, bool hl)
+{
+  vim_snprintf((char *)IObuff, IOSIZE, (char *)message, a1);
+  give_warning(IObuff, hl);
+}
+
 /*
  * Advance msg cursor to column "col".
  */
@@ -2737,7 +2825,6 @@ do_dialog (
                                Ex command */
 )
 {
-  int oldState;
   int retval = 0;
   char_u      *hotkeys;
   int c;
@@ -2750,7 +2837,10 @@ do_dialog (
   }
 
 
-  oldState = State;
+  int save_msg_silent = msg_silent;
+  int oldState = State;
+
+  msg_silent = 0;  // If dialog prompts for input, user needs to see it! #8788
   State = CONFIRM;
   setmouse();
 
@@ -2786,14 +2876,12 @@ do_dialog (
       // Make the character lowercase, as chars in "hotkeys" are.
       c = mb_tolower(c);
       retval = 1;
-      for (i = 0; hotkeys[i]; ++i) {
-        if (has_mbyte) {
-          if ((*mb_ptr2char)(hotkeys + i) == c)
-            break;
-          i += (*mb_ptr2len)(hotkeys + i) - 1;
-        } else if (hotkeys[i] == c)
+      for (i = 0; hotkeys[i]; i++) {
+        if (utf_ptr2char(hotkeys + i) == c) {
           break;
-        ++retval;
+        }
+        i += utfc_ptr2len(hotkeys + i) - 1;
+        retval++;
       }
       if (hotkeys[i])
         break;
@@ -2805,6 +2893,7 @@ do_dialog (
 
   xfree(hotkeys);
 
+  msg_silent = save_msg_silent;
   State = oldState;
   setmouse();
   --no_wait_return;
@@ -2825,25 +2914,13 @@ copy_char (
     int lowercase                  /* make character lower case */
 )
 {
-  int len;
-  int c;
-
-  if (has_mbyte) {
-    if (lowercase) {
-      c = mb_tolower((*mb_ptr2char)(from));
-      return (*mb_char2bytes)(c, to);
-    } else {
-      len = (*mb_ptr2len)(from);
-      memmove(to, from, (size_t)len);
-      return len;
-    }
-  } else {
-    if (lowercase)
-      *to = (char_u)TOLOWER_LOC(*from);
-    else
-      *to = *from;
-    return 1;
+  if (lowercase) {
+    int c = mb_tolower(utf_ptr2char(from));
+    return utf_char2bytes(c, to);
   }
+  int len = utfc_ptr2len(from);
+  memmove(to, from, (size_t)len);
+  return len;
 }
 
 #define HAS_HOTKEY_LEN 30
@@ -2888,7 +2965,7 @@ static char_u * console_dialog_alloc(const char_u *message,
     }
 
     // Advance to the next character
-    mb_ptr_adv(r);
+    MB_PTR_ADV(r);
   }
 
   len += (int)(STRLEN(message)
@@ -3006,7 +3083,7 @@ static void copy_hotkeys_and_msg(const char_u *message, char_u *buttons,
     }
 
     // advance to the next character
-    mb_ptr_adv(r);
+    MB_PTR_ADV(r);
   }
 
   *msgp++ = ':';
@@ -3022,7 +3099,7 @@ void display_confirm_msg(void)
   // Avoid that 'q' at the more prompt truncates the message here.
   confirm_msg_used++;
   if (confirm_msg != NULL) {
-    msg_puts_attr((const char *)confirm_msg, hl_attr(HLF_M));
+    msg_puts_attr((const char *)confirm_msg, HL_ATTR(HLF_M));
   }
   confirm_msg_used--;
 }
