@@ -14,6 +14,7 @@
 #include "nvim/memory.h"
 #include "nvim/message.h"
 #include "nvim/path.h"
+#include "nvim/macros.h"
 #include "nvim/strings.h"
 #include "nvim/eval.h"
 #include "nvim/ex_getln.h"
@@ -55,7 +56,7 @@ int os_setenv(const char *name, const char *value, int overwrite)
   char *envbuf = xmalloc(envbuflen);
   snprintf(envbuf, envbuflen, "%s=%s", name, value);
 
-  WCHAR *p;
+  wchar_t *p;
   utf8_to_utf16(envbuf, &p);
   xfree(envbuf);
   if (p == NULL) {
@@ -145,7 +146,7 @@ void os_get_hostname(char *hostname, size_t size)
     xstrlcpy(hostname, vutsname.nodename, size);
   }
 #elif defined(WIN32)
-  WCHAR host_utf16[MAX_COMPUTERNAME_LENGTH + 1];
+  wchar_t host_utf16[MAX_COMPUTERNAME_LENGTH + 1];
   DWORD host_wsize = sizeof(host_utf16) / sizeof(host_utf16[0]);
   if (GetComputerNameW(host_utf16, &host_wsize) == 0) {
     *hostname = '\0';
@@ -176,7 +177,7 @@ void os_get_hostname(char *hostname, size_t size)
 ///   - do os_dirname() to get the real name of that directory.
 /// This also works with mounts and links.
 /// Don't do this for Windows, it will change the "current dir" for a drive.
-static char_u   *homedir = NULL;
+static char *homedir = NULL;
 
 void init_homedir(void)
 {
@@ -195,16 +196,19 @@ void init_homedir(void)
     const char *homedrive = os_getenv("HOMEDRIVE");
     const char *homepath = os_getenv("HOMEPATH");
     if (homepath == NULL) {
-        homepath = "\\";
+      homepath = "\\";
     }
-    if (homedrive != NULL && strlen(homedrive) + strlen(homepath) < MAXPATHL) {
+    if (homedrive != NULL
+        && strlen(homedrive) + strlen(homepath) < MAXPATHL) {
       snprintf(os_buf, MAXPATHL, "%s%s", homedrive, homepath);
       if (os_buf[0] != NUL) {
         var = os_buf;
-        vim_setenv("HOME", os_buf);
       }
     }
   }
+  if (var == NULL) {
+    var = os_getenv("USERPROFILE");
+  }
 #endif
 
   if (var != NULL) {
@@ -220,7 +224,7 @@ void init_homedir(void)
       }
     }
 #endif
-    homedir = vim_strsave((char_u *)var);
+    homedir = xstrdup(var);
   }
 }
 
@@ -357,7 +361,7 @@ void expand_env_esc(char_u *restrict srcp,
       } else if (src[1] == NUL  // home directory
                  || vim_ispathsep(src[1])
                  || vim_strchr((char_u *)" ,\t\n", src[1]) != NULL) {
-        var = homedir;
+        var = (char_u *)homedir;
         tail = src + 1;
       } else {  // user directory
 #if defined(UNIX)
@@ -372,11 +376,10 @@ void expand_env_esc(char_u *restrict srcp,
           *var++ = *tail++;
         }
         *var = NUL;
-        // Use os_get_user_directory() to get the user directory.
-        // If this function fails, the shell is used to
-        // expand ~user. This is slower and may fail if the shell
-        // does not support ~user (old versions of /bin/sh).
-        var = (char_u *)os_get_user_directory((char *)dst + 1);
+        // Get the user directory. If this fails the shell is used to expand
+        // ~user, which is slower and may fail on old versions of /bin/sh.
+        var = (*dst == NUL) ? NULL
+                            : (char_u *)os_get_user_directory((char *)dst + 1);
         mustfree = true;
         if (var == NULL) {
           expand_T xpc;
@@ -454,12 +457,15 @@ void expand_env_esc(char_u *restrict srcp,
       } else if ((src[0] == ' ' || src[0] == ',') && !one) {
         at_start = true;
       }
-      *dst++ = *src++;
-      --dstlen;
+      if (dstlen > 0) {
+        *dst++ = *src++;
+        dstlen--;
 
-      if (prefix != NULL && src - prefix_len >= srcp
-          && STRNCMP(src - prefix_len, prefix, prefix_len) == 0) {
-        at_start = true;
+        if (prefix != NULL
+            && src - prefix_len >= srcp
+            && STRNCMP(src - prefix_len, prefix, prefix_len) == 0) {
+          at_start = true;
+        }
       }
     }
   }
@@ -608,6 +614,12 @@ char *vim_getenv(const char *name)
     return xstrdup(kos_env_path);
   }
 
+#ifdef WIN32
+  if (strcmp(name, "HOME") == 0) {
+    return xstrdup(homedir);
+  }
+#endif
+
   bool vimruntime = (strcmp(name, "VIMRUNTIME") == 0);
   if (!vimruntime && strcmp(name, "VIM") != 0) {
     return NULL;
@@ -719,108 +731,128 @@ char *vim_getenv(const char *name)
 
 /// Replace home directory by "~" in each space or comma separated file name in
 /// 'src'.
+///
+/// Replace home directory with tilde in each file name
+///
 /// If anything fails (except when out of space) dst equals src.
-/// @param buf When not NULL, check for help files
-/// @param src Input file name
-/// @param dst Where to put the result
-/// @param dstlen Maximum length of the result
-/// @param one If true, only replace one file name, including spaces and commas
-///            in the file name
-void home_replace(const buf_T *const buf, const char_u *src,
-                  char_u *dst, size_t dstlen, bool one)
+///
+/// @param[in]  buf  When not NULL, uses this buffer to check whether it is
+///                  a help file. If it is then path to file is removed
+///                  completely, `one` is ignored and assumed to be true.
+/// @param[in]  src  Input file names. Assumed to be a space/comma separated
+///                  list unless `one` is true.
+/// @param[out]  dst  Where to put the result.
+/// @param[in]  dstlen  Destination length.
+/// @param[in]  one  If true, assumes source is a single file name and not
+///                  a list of them.
+///
+/// @return length of the string put into dst, does not include NUL byte.
+size_t home_replace(const buf_T *const buf, const char_u *src,
+                    char_u *const dst, size_t dstlen, const bool one)
+  FUNC_ATTR_NONNULL_ARG(3)
 {
-  size_t dirlen = 0, envlen = 0;
-  size_t len;
+  size_t dirlen = 0;
+  size_t envlen = 0;
 
   if (src == NULL) {
     *dst = NUL;
-    return;
+    return 0;
   }
 
-  /*
-   * If the file is a help file, remove the path completely.
-   */
   if (buf != NULL && buf->b_help) {
-    xstrlcpy((char *)dst, (char *)path_tail(src), dstlen);
-    return;
+    const size_t dlen = xstrlcpy((char *)dst, (char *)path_tail(src), dstlen);
+    return MIN(dlen, dstlen - 1);
   }
 
-  /*
-   * We check both the value of the $HOME environment variable and the
-   * "real" home directory.
-   */
-  if (homedir != NULL)
-    dirlen = STRLEN(homedir);
+  // We check both the value of the $HOME environment variable and the
+  // "real" home directory.
+  if (homedir != NULL) {
+    dirlen = strlen(homedir);
+  }
 
-  char_u *homedir_env = (char_u *)os_getenv("HOME");
+  const char *homedir_env = os_getenv("HOME");
+#ifdef WIN32
+  if (homedir_env == NULL) {
+    homedir_env = os_getenv("USERPROFILE");
+  }
+#endif
+  char *homedir_env_mod = (char *)homedir_env;
   bool must_free = false;
 
-  if (homedir_env != NULL && vim_strchr(homedir_env, '~') != NULL) {
+  if (homedir_env_mod != NULL && *homedir_env_mod == '~') {
     must_free = true;
     size_t usedlen = 0;
-    size_t flen = STRLEN(homedir_env);
+    size_t flen = strlen(homedir_env_mod);
     char_u *fbuf = NULL;
-    (void)modify_fname((char_u *)":p", &usedlen, &homedir_env, &fbuf, &flen);
-    flen = STRLEN(homedir_env);
-    if (flen > 0 && vim_ispathsep(homedir_env[flen - 1]))
-      /* Remove the trailing / that is added to a directory. */
-      homedir_env[flen - 1] = NUL;
+    (void)modify_fname((char_u *)":p", &usedlen, (char_u **)&homedir_env_mod,
+                       &fbuf, &flen);
+    flen = strlen(homedir_env_mod);
+    assert(homedir_env_mod != homedir_env);
+    if (vim_ispathsep(homedir_env_mod[flen - 1])) {
+      // Remove the trailing / that is added to a directory.
+      homedir_env_mod[flen - 1] = NUL;
+    }
   }
 
-  if (homedir_env != NULL)
-    envlen = STRLEN(homedir_env);
+  if (homedir_env_mod != NULL) {
+    envlen = strlen(homedir_env_mod);
+  }
 
-  if (!one)
+  if (!one) {
     src = skipwhite(src);
+  }
+  char *dst_p = (char *)dst;
   while (*src && dstlen > 0) {
-    /*
-     * Here we are at the beginning of a file name.
-     * First, check to see if the beginning of the file name matches
-     * $HOME or the "real" home directory. Check that there is a '/'
-     * after the match (so that if e.g. the file is "/home/pieter/bla",
-     * and the home directory is "/home/piet", the file does not end up
-     * as "~er/bla" (which would seem to indicate the file "bla" in user
-     * er's home directory)).
-     */
-    char_u *p = homedir;
-    len = dirlen;
-    for (;; ) {
-      if (   len
-             && fnamencmp(src, p, len) == 0
-             && (vim_ispathsep(src[len])
-                 || (!one && (src[len] == ',' || src[len] == ' '))
-                 || src[len] == NUL)) {
+    // Here we are at the beginning of a file name.
+    // First, check to see if the beginning of the file name matches
+    // $HOME or the "real" home directory. Check that there is a '/'
+    // after the match (so that if e.g. the file is "/home/pieter/bla",
+    // and the home directory is "/home/piet", the file does not end up
+    // as "~er/bla" (which would seem to indicate the file "bla" in user
+    // er's home directory)).
+    char *p = homedir;
+    size_t len = dirlen;
+    for (;;) {
+      if (len
+          && fnamencmp(src, (char_u *)p, len) == 0
+          && (vim_ispathsep(src[len])
+              || (!one && (src[len] == ',' || src[len] == ' '))
+              || src[len] == NUL)) {
         src += len;
-        if (--dstlen > 0)
-          *dst++ = '~';
+        if (--dstlen > 0) {
+          *dst_p++ = '~';
+        }
 
-        /*
-         * If it's just the home directory, add  "/".
-         */
-        if (!vim_ispathsep(src[0]) && --dstlen > 0)
-          *dst++ = '/';
+        // If it's just the home directory, add  "/".
+        if (!vim_ispathsep(src[0]) && --dstlen > 0) {
+          *dst_p++ = '/';
+        }
         break;
       }
-      if (p == homedir_env)
+      if (p == homedir_env_mod) {
         break;
-      p = homedir_env;
+      }
+      p = homedir_env_mod;
       len = envlen;
     }
 
-    /* if (!one) skip to separator: space or comma */
-    while (*src && (one || (*src != ',' && *src != ' ')) && --dstlen > 0)
-      *dst++ = *src++;
-    /* skip separator */
-    while ((*src == ' ' || *src == ',') && --dstlen > 0)
-      *dst++ = *src++;
+    // if (!one) skip to separator: space or comma.
+    while (*src && (one || (*src != ',' && *src != ' ')) && --dstlen > 0) {
+      *dst_p++ = (char)(*src++);
+    }
+    // Skip separator.
+    while ((*src == ' ' || *src == ',') && --dstlen > 0) {
+      *dst_p++ = (char)(*src++);
+    }
   }
-  /* if (dstlen == 0) out of space, what to do??? */
+  // If (dstlen == 0) out of space, what to do???
 
-  *dst = NUL;
+  *dst_p = NUL;
 
   if (must_free) {
-    xfree(homedir_env);
+    xfree(homedir_env_mod);
   }
+  return (size_t)(dst_p - (char *)dst);
 }
 
 /// Like home_replace, store the replaced string in allocated memory.
@@ -886,8 +918,8 @@ bool os_setenv_append_path(const char *fname)
 // No prescribed maximum on unix.
 # define MAX_ENVPATHLEN INT_MAX
 #endif
-  if (!path_is_absolute_path((char_u *)fname)) {
-    EMSG2(_(e_intern2), "os_setenv_append_path()");
+  if (!path_is_absolute((char_u *)fname)) {
+    internal_error("os_setenv_append_path()");
     return false;
   }
   const char *tail = (char *)path_tail_with_sep((char_u *)fname);
@@ -913,31 +945,6 @@ bool os_setenv_append_path(const char *fname)
   return false;
 }
 
-/// Returns true if the terminal can be assumed to silently ignore unknown
-/// control codes.
-bool os_term_is_nice(void)
-{
-#if defined(__APPLE__) || defined(WIN32)
-  return true;
-#else
-  const char *vte_version = os_getenv("VTE_VERSION");
-  if ((vte_version && atoi(vte_version) >= 3900)
-      || os_getenv("KONSOLE_PROFILE_NAME")
-      || os_getenv("KONSOLE_DBUS_SESSION")) {
-    return true;
-  }
-  const char *termprg = os_getenv("TERM_PROGRAM");
-  if (termprg && striequal(termprg, "iTerm.app")) {
-    return true;
-  }
-  const char *term = os_getenv("TERM");
-  if (term && strncmp(term, "rxvt", 4) == 0) {
-    return true;
-  }
-  return false;
-#endif
-}
-
 /// Returns true if `sh` looks like it resolves to "cmd.exe".
 bool os_shell_is_cmdexe(const char *sh)
   FUNC_ATTR_NONNULL_ALL
