@@ -13,13 +13,6 @@
 # include "nvim/os/unix_defs.h"
 #endif
 
-/// File descriptor number used for standard IO streams
-enum {
-  OS_STDIN_FILENO = STDIN_FILENO,
-  OS_STDOUT_FILENO = STDOUT_FILENO,
-  OS_STDERR_FILENO = STDERR_FILENO,
-};
-
 #define BASENAMELEN (NAME_MAX - 5)
 
 // Use the system path length if it makes sense.
@@ -32,15 +25,6 @@ enum {
 // Command-processing buffer. Use large buffers for all platforms.
 #define CMDBUFFSIZE 1024
 
-// Use up to 5 Mbyte for a buffer.
-#ifndef DFLT_MAXMEM
-# define DFLT_MAXMEM (5 * 1024)
-#endif
-// use up to 10 Mbyte for Vim.
-#ifndef DFLT_MAXMEMTOT
-# define DFLT_MAXMEMTOT (10 * 1024)
-#endif
-
 // Note: Some systems need both string.h and strings.h (Savage).  However,
 // some systems can't handle both, only use string.h in that case.
 #include <string.h>
