@@ -14,6 +14,8 @@
 #include "nvim/option_defs.h"
 #include "nvim/ui.h"
 #include "nvim/os/input.h"
+#include "nvim/ex_docmd.h"
+#include "nvim/edit.h"
 
 #ifdef INCLUDE_GENERATED_DECLARATIONS
 # include "state.c.generated.h"
@@ -62,6 +64,12 @@ getkey:
       may_sync_undo();
     }
 
+#if MIN_LOG_LEVEL <= DEBUG_LOG_LEVEL
+    char *keyname = key == K_EVENT
+                    ? "K_EVENT" : (char *)get_special_key_name(key, mod_mask);
+    DLOG("input: %s", keyname);
+#endif
+
     int execute_result = s->execute(s, key);
     if (!execute_result) {
       break;
@@ -71,13 +79,13 @@ getkey:
   }
 }
 
-/// Return TRUE if in the current mode we need to use virtual.
-int virtual_active(void)
+/// Return true if in the current mode we need to use virtual.
+bool virtual_active(void)
 {
   // While an operator is being executed we return "virtual_op", because
   // VIsual_active has already been reset, thus we can't check for "block"
   // being used.
-  if (virtual_op != MAYBE) {
+  if (virtual_op != kNone) {
     return virtual_op;
   }
   return ve_flags == VE_ALL
@@ -127,19 +135,25 @@ char *get_mode(void)
     if (State & VREPLACE_FLAG) {
       buf[0] = 'R';
       buf[1] = 'v';
-    } else if (State & REPLACE_FLAG) {
-      buf[0] = 'R';
     } else {
-      buf[0] = 'i';
+      if (State & REPLACE_FLAG) {
+        buf[0] = 'R';
+      } else {
+        buf[0] = 'i';
+      }
+      if (ins_compl_active()) {
+        buf[1] = 'c';
+      } else if (ctrl_x_mode == 1) {
+        buf[1] = 'x';
+      }
     }
-  } else if (State & CMDLINE) {
+  } else if ((State & CMDLINE) || exmode_active) {
     buf[0] = 'c';
-    if (exmode_active) {
+    if (exmode_active == EXMODE_VIM) {
       buf[1] = 'v';
+    } else if (exmode_active == EXMODE_NORMAL) {
+      buf[1] = 'e';
     }
-  } else if (exmode_active) {
-    buf[0] = 'c';
-    buf[1] = 'e';
   } else if (State & TERM_FOCUS) {
     buf[0] = 't';
   } else {
