@@ -50,6 +50,13 @@
 #include "nvim/os/shell.h"
 #include "nvim/eval/encode.h"
 
+#ifdef __MINGW32__
+# undef fpclassify
+# define fpclassify __fpclassify
+# undef isnan
+# define isnan _isnan
+#endif
+
 /*
  * Copy "string" into newly allocated memory.
  */
@@ -200,7 +207,7 @@ char_u *vim_strsave_shellescape(const char_u *string,
 
   /* First count the number of extra bytes required. */
   size_t length = STRLEN(string) + 3;       // two quotes and a trailing NUL
-  for (const char_u *p = string; *p != NUL; mb_ptr_adv(p)) {
+  for (const char_u *p = string; *p != NUL; MB_PTR_ADV(p)) {
 #ifdef WIN32
     if (!p_ssl) {
       if (*p == '"') {
@@ -337,14 +344,17 @@ char *strcase_save(const char *const orig, bool upper)
 
   char *p = res;
   while (*p != NUL) {
-    int l;
-
     int c = utf_ptr2char((const char_u *)p);
+    int l = utf_ptr2len((const char_u *)p);
+    if (c == 0) {
+      // overlong sequence, use only the first byte
+      c = *p;
+      l = 1;
+    }
     int uc = upper ? mb_toupper(c) : mb_tolower(c);
 
     // Reallocate string when byte count changes.  This is rare,
     // thus it's OK to do another malloc()/free().
-    l = utf_ptr2len((const char_u *)p);
     int newl = utf_char2len(uc);
     if (newl != l) {
       // TODO(philix): use xrealloc() in strup_save()
@@ -448,25 +458,6 @@ char_u *vim_strchr(const char_u *const string, const int c)
   }
 }
 
-/*
- * Search for last occurrence of "c" in "string".
- * Return NULL if not found.
- * Does not handle multi-byte char for "c"!
- */
-char_u *vim_strrchr(const char_u *string, int c)
-  FUNC_ATTR_NONNULL_ALL FUNC_ATTR_PURE
-{
-  const char_u *retval = NULL;
-  const char_u *p = string;
-
-  while (*p) {
-    if (*p == c)
-      retval = p;
-    mb_ptr_adv(p);
-  }
-  return (char_u *) retval;
-}
-
 /*
  * Sort an array of strings.
  */
@@ -702,6 +693,7 @@ static float_T tv_float(typval_T *const tvs, int *const idxp)
 ///
 /// @see vim_vsnprintf().
 int vim_snprintf_add(char *str, size_t str_m, char *fmt, ...)
+  FUNC_ATTR_PRINTF(3, 4)
 {
   const size_t len = strlen(str);
   size_t space;
@@ -727,6 +719,7 @@ int vim_snprintf_add(char *str, size_t str_m, char *fmt, ...)
 /// @return Number of bytes excluding NUL byte that would be written to the
 ///         string if str_m was greater or equal to the return value.
 int vim_snprintf(char *str, size_t str_m, const char *fmt, ...)
+  FUNC_ATTR_PRINTF(3, 4)
 {
   va_list ap;
   va_start(ap, fmt);
@@ -1006,10 +999,13 @@ int vim_vsnprintf(char *str, size_t str_m, const char *fmt, va_list ap,
           } else if (fmt_spec == 'd') {
             // signed
             switch (length_modifier) {
-              case '\0':
+              case '\0': {
+                arg = (int)(tvs ? tv_nr(tvs, &arg_idx) : va_arg(ap, int));
+                break;
+              }
               case 'h': {
-                // char and short arguments are passed as int
-                arg = (tvs ? (int)tv_nr(tvs, &arg_idx) : va_arg(ap, int));
+                // char and short arguments are passed as int16_t
+                arg = (int16_t)(tvs ? tv_nr(tvs, &arg_idx) : va_arg(ap, int));
                 break;
               }
               case 'l': {
@@ -1038,11 +1034,16 @@ int vim_vsnprintf(char *str, size_t str_m, const char *fmt, va_list ap,
           } else {
             // unsigned
             switch (length_modifier) {
-              case '\0':
+              case '\0': {
+                uarg = (unsigned int)(tvs
+                                      ? tv_nr(tvs, &arg_idx)
+                                      : va_arg(ap, unsigned int));
+                break;
+              }
               case 'h': {
-                uarg = (tvs
-                        ? (unsigned)tv_nr(tvs, &arg_idx)
-                        : va_arg(ap, unsigned));
+                uarg = (uint16_t)(tvs
+                                  ? tv_nr(tvs, &arg_idx)
+                                  : va_arg(ap, unsigned int));
                 break;
               }
               case 'l': {
@@ -1138,8 +1139,8 @@ int vim_vsnprintf(char *str, size_t str_m, const char *fmt, va_list ap,
                                               f, uarg);
                 break;
               }
-              assert(str_arg_l < sizeof(tmp));
             }
+            assert(str_arg_l < sizeof(tmp));
 
             // include the optional minus sign and possible "0x" in the region
             // before the zero padding insertion point
@@ -1226,6 +1227,7 @@ int vim_vsnprintf(char *str, size_t str_m, const char *fmt, va_list ap,
               str_arg_l = 3;
               zero_padding = 0;
             } else {
+              // Regular float number
               format[0] = '%';
               size_t l = 1;
               if (force_sign) {
@@ -1246,11 +1248,10 @@ int vim_vsnprintf(char *str, size_t str_m, const char *fmt, va_list ap,
               }
 
               // Cast to char to avoid a conversion warning on Ubuntu 12.04.
+              assert(l + 1 < sizeof(format));
               format[l] = (char)(fmt_spec == 'F' ? 'f' : fmt_spec);
               format[l + 1] = NUL;
 
-              // Regular float number
-              assert(l + 1 < sizeof(format));
               str_arg_l = (size_t)snprintf(tmp, sizeof(tmp), format, f);
               assert(str_arg_l < sizeof(tmp));
 
@@ -1369,16 +1370,14 @@ int vim_vsnprintf(char *str, size_t str_m, const char *fmt, va_list ap,
         }
 
         // insert zero padding as requested by precision or min field width
-        if (number_of_zeros_to_pad > 0) {
-          size_t zn = number_of_zeros_to_pad;
-          if (str_avail) {
-            size_t avail = str_m - str_l;
-            memset(str + str_l, '0', MIN(zn, avail));
-            str_avail = zn < avail;
-          }
-          assert(zn <= SIZE_MAX - str_l);
-          str_l += zn;
+        size_t zn = number_of_zeros_to_pad;
+        if (str_avail) {
+          size_t avail = str_m - str_l;
+          memset(str + str_l, '0', MIN(zn, avail));
+          str_avail = zn < avail;
         }
+        assert(zn <= SIZE_MAX - str_l);
+        str_l += zn;
       }
 
       // insert formatted string
