@@ -34,7 +34,7 @@ describe('api/tabpage', function()
       eq(1, funcs.exists('t:lua'))
       curtabmeths.del_var('lua')
       eq(0, funcs.exists('t:lua'))
-      eq({false, 'Key does not exist: lua'}, meth_pcall(curtabmeths.del_var, 'lua'))
+      eq({false, 'Key not found: lua'}, meth_pcall(curtabmeths.del_var, 'lua'))
       curtabmeths.set_var('lua', 1)
       command('lockvar t:lua')
       eq({false, 'Key is locked: lua'}, meth_pcall(curtabmeths.del_var, 'lua'))
