@@ -17,6 +17,7 @@ local pathroot = helpers.pathroot
 local nvim_set = helpers.nvim_set
 local expect_twostreams = helpers.expect_twostreams
 local expect_msg_seq = helpers.expect_msg_seq
+local expect_err = helpers.expect_err
 local Screen = require('test.functional.ui.screen')
 
 -- Kill process with given pid
@@ -115,6 +116,17 @@ describe('jobs', function()
     ok(string.find(err, "E475: Invalid argument: expected valid directory$") ~= nil)
   end)
 
+  it('produces error when using non-executable `cwd`', function()
+    if iswin() then return end  -- N/A for Windows
+
+    local dir = 'Xtest_not_executable_dir'
+    mkdir(dir)
+    funcs.setfperm(dir, 'rw-------')
+    expect_err('E475: Invalid argument: expected valid directory$', nvim,
+               'command', "call jobstart('pwd', {'cwd': '" .. dir .. "'})")
+    rmdir(dir)
+  end)
+
   it('returns 0 when it fails to start', function()
     eq("", eval("v:errmsg"))
     feed_command("let g:test_jobid = jobstart([])")
@@ -417,7 +429,7 @@ describe('jobs', function()
         \ })
     ]])
 
-    screen:expect("{2:E120: Using <SID> not in a script context: s:OnEvent}",nil,nil,nil,true)
+    screen:expect{any="{2:E120: Using <SID> not in a script context: s:OnEvent}"}
   end)
 
   it('does not repeat output with slow output handlers', function()
@@ -663,7 +675,7 @@ describe('jobs', function()
         call rpcnotify(g:channel, 'wait', jobwait([
         \  jobstart('exit 4'),
         \  jobstart((has('win32') ? 'Start-Sleep 10' : 'sleep 10').'; exit 5'),
-        \  ], has('win32') ? 3000 : 100))
+        \  ], has('win32') ? 6000 : 100))
         ]])
         eq({'notification', 'wait', {{4, -1}}}, next_msg())
       end)
@@ -731,7 +743,7 @@ describe('jobs', function()
     local children
     retry(nil, nil, function()
       children = meths.get_proc_children(ppid)
-      eq(3, #children)
+      eq((iswin() and 4 or 3), #children)
     end)
     -- Assert that nvim_get_proc() sees the children.
     for _, child_pid in ipairs(children) do
