@@ -8,6 +8,7 @@ local eval = helpers.eval
 local funcs = helpers.funcs
 local insert = helpers.insert
 local exc_exec = helpers.exc_exec
+local source = helpers.source
 local Screen = require('test.functional.ui.screen')
 
 describe('mappings with <Cmd>', function()
@@ -28,6 +29,9 @@ describe('mappings with <Cmd>', function()
       [5] = {background = Screen.colors.LightGrey},
       [6] = {foreground = Screen.colors.Blue1},
       [7] = {bold = true, reverse = true},
+      [8] = {background = Screen.colors.WebGray},
+      [9] = {background = Screen.colors.LightMagenta},
+      [10] = {foreground = Screen.colors.Red},
     })
     screen:attach()
 
@@ -655,8 +659,38 @@ describe('mappings with <Cmd>', function()
     eq('n', eval('mode(1)'))
   end)
 
+  it('works in insert completion (Ctrl-X) mode', function()
+    feed('os<c-x><c-n>')
+    screen:expect([[
+      some short lines                                                 |
+      some^                                                             |
+      {8:some           }                                                  |
+      {9:short          }{1:                                                  }|
+      {1:~                                                                }|
+      {1:~                                                                }|
+      {1:~                                                                }|
+      {4:-- Keyword Local completion (^N^P) }{3:match 1 of 2}                  |
+    ]])
+
+    feed('<f3>')
+    eq('ic', eval('m'))
+
+    -- ensure a redraw, this would have moved the cursor
+    -- instead if CTRL-X mode was left.
+    feed('<up>')
+    screen:expect([[
+      some short lines                                                 |
+      some^                                                             |
+      {9:some           }                                                  |
+      {9:short          }{1:                                                  }|
+      {1:~                                                                }|
+      {1:~                                                                }|
+      {1:~                                                                }|
+      {4:-- Keyword Local completion (^N^P) }{10:Back at original}              |
+    ]])
+  end)
+
   it('works in cmdline mode', function()
-    cmdmap('<F2>', 'call setcmdpos(2)')
     feed(':text<F3>')
     eq('c', eval('m'))
     -- didn't leave cmdline mode
@@ -768,5 +802,43 @@ describe('mappings with <Cmd>', function()
 
   end)
 
+  it("doesn't crash when invoking cmdline mode recursively #8859", function()
+    cmdmap('<F2>', 'norm! :foo')
+    feed(':bar')
+    screen:expect([[
+      some short lines                                                 |
+      of test text                                                     |
+      {1:~                                                                }|
+      {1:~                                                                }|
+      {1:~                                                                }|
+      {1:~                                                                }|
+      {1:~                                                                }|
+      :bar^                                                             |
+    ]])
+
+    feed('<f2>x')
+    screen:expect([[
+      some short lines                                                 |
+      of test text                                                     |
+      {1:~                                                                }|
+      {1:~                                                                }|
+      {1:~                                                                }|
+      {1:~                                                                }|
+      {1:~                                                                }|
+      :barx^                                                            |
+    ]])
+  end)
+
+  it("works with <SID> mappings", function()
+    source([[
+      map <f2> <Cmd>call <SID>do_it()<Cr>
+      function! s:do_it()
+        let g:x = 10
+      endfunction
+    ]])
+    feed('<f2>')
+    eq('', eval('v:errmsg'))
+    eq(10, eval('g:x'))
+  end)
 end)
 
