@@ -9,7 +9,7 @@ local eval = helpers.eval
 local shada_file = 'Xtest.shada'
 
 local function _clear()
-  set_session(spawn({nvim_prog, '--embed', '-u', 'NONE',
+  set_session(spawn({nvim_prog, '--embed', '--headless', '-u', 'NONE',
                      -- Need shada for these tests.
                      '-i', shada_file,
                      '--cmd', 'set noswapfile undodir=. directory=. viewdir=. backupdir=. belloff= noshowcmd noruler'}))
@@ -29,6 +29,7 @@ describe(':oldfiles', function()
   it('shows most recently used files', function()
     local screen = Screen.new(100, 5)
     screen:attach()
+    feed_command("set display-=msgsep")
     feed_command('edit testfile1')
     feed_command('edit testfile2')
     feed_command('wshada')
