@@ -137,7 +137,7 @@ describe('ShaDa support code', function()
 
   it('does not write NONE file', function()
     local session = spawn({nvim_prog, '-u', 'NONE', '-i', 'NONE', '--embed',
-                           '--cmd', 'qall'}, true)
+                           '--headless', '--cmd', 'qall'}, true)
     session:close()
     eq(nil, lfs.attributes('NONE'))
     eq(nil, lfs.attributes('NONE.tmp.a'))
@@ -145,8 +145,8 @@ describe('ShaDa support code', function()
 
   it('does not read NONE file', function()
     write_file('NONE', '\005\001\015\131\161na\162rX\194\162rc\145\196\001-')
-    local session = spawn({nvim_prog, '-u', 'NONE', '-i', 'NONE', '--embed'},
-                          true)
+    local session = spawn({nvim_prog, '-u', 'NONE', '-i', 'NONE', '--embed',
+                           '--headless'}, true)
     set_session(session)
     eq('', funcs.getreg('a'))
     session:close()
@@ -181,13 +181,13 @@ describe('ShaDa support code', function()
     nvim_command('set shada+=%')
     nvim_command('wshada! ' .. shada_fname)
     local readme_fname = funcs.resolve(paths.test_source_path) .. '/README.md'
-    eq({[7]=1, [8]=2, [9]=1, [10]=4, [11]=1}, find_file(readme_fname))
+    eq({[7]=2, [8]=2, [9]=1, [10]=4, [11]=1}, find_file(readme_fname))
     nvim_command('set shada+=r~')
     nvim_command('wshada! ' .. shada_fname)
     eq({}, find_file(readme_fname))
     nvim_command('set shada-=r~')
     nvim_command('wshada! ' .. shada_fname)
-    eq({[7]=1, [8]=2, [9]=1, [10]=4, [11]=1}, find_file(readme_fname))
+    eq({[7]=2, [8]=2, [9]=1, [10]=4, [11]=1}, find_file(readme_fname))
     nvim_command('set shada+=r' .. funcs.escape(
       funcs.escape(paths.test_source_path, '$~'), ' "\\,'))
     nvim_command('wshada! ' .. shada_fname)
@@ -206,7 +206,7 @@ describe('ShaDa support code', function()
     nvim_command('undo')
     nvim_command('set shada+=%')
     nvim_command('wshada! ' .. shada_fname)
-    eq({[7]=1, [8]=2, [9]=1, [10]=4, [11]=2}, find_file(fname))
+    eq({[7]=2, [8]=2, [9]=1, [10]=4, [11]=2}, find_file(fname))
     nvim_command('set shada+=r' .. pwd .. '/АБВ')
     nvim_command('wshada! ' .. shada_fname)
     eq({}, find_file(fname))
