@@ -24,6 +24,7 @@ before_each(function()
   clear()
   screen = Screen.new(40, 8)
   screen:attach()
+  command("set display-=msgsep")
   source([[
     highlight RBP1 guibg=Red
     highlight RBP2 guibg=Yellow
@@ -31,7 +32,7 @@ before_each(function()
     highlight RBP4 guibg=Blue
     let g:NUM_LVLS = 4
     function Redraw()
-      redraw!
+      mode
       return ''
     endfunction
     let g:id = ''
@@ -144,7 +145,13 @@ before_each(function()
     EOB={bold = true, foreground = Screen.colors.Blue1},
     ERR={foreground = Screen.colors.Grey100, background = Screen.colors.Red},
     SK={foreground = Screen.colors.Blue},
-    PE={bold = true, foreground = Screen.colors.SeaGreen4}
+    PE={bold = true, foreground = Screen.colors.SeaGreen4},
+    NUM={foreground = Screen.colors.Blue2},
+    NPAR={foreground = Screen.colors.Yellow},
+    SQ={foreground = Screen.colors.Blue3},
+    SB={foreground = Screen.colors.Blue4},
+    E={foreground = Screen.colors.Red, background = Screen.colors.Blue},
+    M={bold = true},
   })
 end)
 
@@ -260,7 +267,7 @@ describe('Command-line coloring', function()
       :echo {RBP1:(}{RBP2:(}42{RBP2:)}^                             |
     ]])
     redraw_input()
-    screen:expect([[
+    screen:expect{grid=[[
                                               |
       {EOB:~                                       }|
       {EOB:~                                       }|
@@ -269,7 +276,7 @@ describe('Command-line coloring', function()
       {EOB:~                                       }|
       {EOB:~                                       }|
       :echo {RBP1:(}{RBP2:(}42{RBP2:)}^                             |
-    ]])
+    ]], reset=true}
   end)
   for _, func_part in ipairs({'', 'n', 'msg'}) do
     it('disables :echo' .. func_part .. ' messages', function()
@@ -487,7 +494,7 @@ describe('Command-line coloring', function()
       {EOB:~                                       }|
       {EOB:~                                       }|
       {EOB:~                                       }|
-      Type  :quit<Enter>  to exit Nvim        |
+      Type  :qa!  and pr...nges and exit Nvim |
     ]])
   end)
   it('works fine with NUL, NL, CR', function()
@@ -731,8 +738,24 @@ describe('Command-line coloring', function()
     feed('<CR><CR>')
     eq('', meths.get_var('out'))
   end)
+  it('does not crash when callback has caught not-a-editor-command exception',
+  function()
+    source([[
+      function CaughtExc(cmdline) abort
+        try
+          gibberish
+        catch
+          " Do nothing
+        endtry
+        return []
+      endfunction
+    ]])
+    set_color_cb('CaughtExc')
+    start_prompt('1')
+    eq(1, meths.eval('1'))
+  end)
 end)
-describe('Ex commands coloring support', function()
+describe('Ex commands coloring', function()
   it('works', function()
     meths.set_var('Nvim_color_cmdline', 'RainBowParens')
     feed(':echo (((1)))')
@@ -808,7 +831,7 @@ describe('Ex commands coloring support', function()
                                               |
     ]])
   end)
-  it('does not prevent mapping error from cancelling prompt', function()
+  it('mapping error does not cancel prompt', function()
     command("cnoremap <expr> x execute('throw 42')[-1]")
     feed(':#x')
     screen:expect([[
@@ -823,27 +846,17 @@ describe('Ex commands coloring support', function()
     ]])
     feed('<CR>')
     screen:expect([[
-      ^                                        |
       {EOB:~                                       }|
       {EOB:~                                       }|
       {EOB:~                                       }|
-      {EOB:~                                       }|
-      {EOB:~                                       }|
-      {EOB:~                                       }|
-                                              |
+      :#                                      |
+      {ERR:Error detected while processing :}       |
+      {ERR:E605: Exception not caught: 42}          |
+      {ERR:E749: empty buffer}                      |
+      {PE:Press ENTER or type command to continue}^ |
     ]])
     feed('<CR>')
-    screen:expect([[
-      ^                                        |
-      {EOB:~                                       }|
-      {EOB:~                                       }|
-      {EOB:~                                       }|
-      {EOB:~                                       }|
-      {EOB:~                                       }|
-      {EOB:~                                       }|
-                                              |
-    ]])
-    eq('\nError detected while processing :\nE605: Exception not caught: 42',
+    eq('Error detected while processing :\nE605: Exception not caught: 42\nE749: empty buffer',
        meths.command_output('messages'))
   end)
   it('errors out when failing to get callback', function()
@@ -863,7 +876,10 @@ describe('Ex commands coloring support', function()
 end)
 describe('Expressions coloring support', function()
   it('works', function()
-    meths.set_var('Nvim_color_expr', 'RainBowParens')
+    meths.command('hi clear NvimNumber')
+    meths.command('hi clear NvimNestingParenthesis')
+    meths.command('hi NvimNumber guifg=Blue2')
+    meths.command('hi NvimNestingParenthesis guifg=Yellow')
     feed(':echo <C-r>=(((1)))')
     screen:expect([[
                                               |
@@ -873,21 +889,103 @@ describe('Expressions coloring support', function()
       {EOB:~                                       }|
       {EOB:~                                       }|
       {EOB:~                                       }|
-      ={RBP1:(}{RBP2:(}{RBP3:(}1{RBP3:)}{RBP2:)}{RBP1:)}^                                |
+      ={NPAR:(((}{NUM:1}{NPAR:)))}^                                |
     ]])
   end)
-  it('errors out when failing to get callback', function()
+  it('does not use Nvim_color_expr', function()
     meths.set_var('Nvim_color_expr', 42)
+    -- Used to error out due to failing to get callback.
+    meths.command('hi clear NvimNumber')
+    meths.command('hi NvimNumber guifg=Blue2')
     feed(':<C-r>=1')
     screen:expect([[
+                                              |
       {EOB:~                                       }|
       {EOB:~                                       }|
       {EOB:~                                       }|
-      =                                       |
-      {ERR:E5409: Unable to get g:Nvim_color_expr c}|
-      {ERR:allback: Vim:E6000: Argument is not a fu}|
-      {ERR:nction or function name}                 |
-      =1^                                      |
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      ={NUM:1}^                                      |
+    ]])
+  end)
+  it('works correctly with non-ASCII and control characters', function()
+    meths.command('hi clear NvimStringBody')
+    meths.command('hi clear NvimStringQuote')
+    meths.command('hi clear NvimInvalid')
+    meths.command('hi NvimStringQuote guifg=Blue3')
+    meths.command('hi NvimStringBody guifg=Blue4')
+    meths.command('hi NvimInvalid guifg=Red guibg=Blue')
+    feed('i<C-r>="«»"«»')
+    screen:expect([[
+                                              |
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      ={SQ:"}{SB:«»}{SQ:"}{E:«»}^                                 |
+    ]])
+    feed('<C-c>')
+    screen:expect([[
+      ^                                        |
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {M:-- INSERT --}                            |
+    ]])
+    feed('<Esc>')
+    screen:expect([[
+      ^                                        |
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+                                              |
+    ]])
+    feed(':<C-\\>e"<C-v><C-x>"<C-v><C-x>')
+    -- TODO(ZyX-I): Parser highlighting should not override special character
+    --              highlighting.
+    screen:expect([[
+                                              |
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      ={SQ:"}{SB:^X}{SQ:"}{ERR:^X}^                                 |
+    ]])
+    feed('<C-c>')
+    screen:expect([[
+                                              |
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      :^                                       |
+    ]])
+    funcs.setreg('a', {'\192'})
+    feed('<C-r>="<C-r><C-r>a"<C-r><C-r>a"foo"')
+    -- TODO(ZyX-I): Parser highlighting should not override special character
+    --              highlighting.
+    screen:expect([[
+                                              |
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      {EOB:~                                       }|
+      ={SQ:"}{SB:<c0>}{SQ:"}{E:<c0>"}{SB:foo}{E:"}^                        |
     ]])
   end)
 end)
