@@ -48,13 +48,13 @@ describe('screen', function()
   end)
 end)
 
-describe('Screen', function()
+local function screen_tests(linegrid)
   local screen
 
   before_each(function()
     clear()
     screen = Screen.new()
-    screen:attach()
+    screen:attach({rgb=true,ext_linegrid=linegrid})
     screen:set_default_attr_ids( {
       [0] = {bold=true, foreground=255},
       [1] = {bold=true, reverse=true},
@@ -76,11 +76,26 @@ describe('Screen', function()
       local function check()
         eq(true, screen.suspended)
       end
+
+      command('let g:ev = []')
+      command('autocmd VimResume  * :call add(g:ev, "r")')
+      command('autocmd VimSuspend * :call add(g:ev, "s")')
+
+      eq(false, screen.suspended)
       command('suspend')
+      eq({ 's', 'r' }, eval('g:ev'))
+
       screen:expect(check)
       screen.suspended = false
+
       feed('<c-z>')
+      eq({ 's', 'r', 's', 'r' }, eval('g:ev'))
+
       screen:expect(check)
+      screen.suspended = false
+
+      command('suspend')
+      eq({ 's', 'r', 's', 'r', 's', 'r' }, eval('g:ev'))
     end)
   end)
 
@@ -141,6 +156,82 @@ describe('Screen', function()
     end)
   end)
 
+  describe('statusline', function()
+    it('is redrawn after <c-l>', function()
+      command('set laststatus=2')
+      screen:expect([[
+        ^                                                     |
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {1:[No Name]                                            }|
+                                                             |
+      ]])
+
+      feed('<c-l>')
+      screen:expect{grid=[[
+        ^                                                     |
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {1:[No Name]                                            }|
+                                                             |
+      ]], reset=true}
+
+      command('split')
+      screen:expect([[
+        ^                                                     |
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {1:[No Name]                                            }|
+                                                             |
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {3:[No Name]                                            }|
+                                                             |
+      ]])
+
+      feed('<c-l>')
+      screen:expect{grid=[[
+        ^                                                     |
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {1:[No Name]                                            }|
+                                                             |
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {3:[No Name]                                            }|
+                                                             |
+      ]], reset=true}
+    end)
+  end)
+
   describe('window', function()
     describe('split', function()
       it('horizontal', function()
@@ -189,12 +280,12 @@ describe('Screen', function()
         command('vsp')
         command('vsp')
         screen:expect([[
-          ^                    {3:|}                {3:|}               |
-          {0:~                   }{3:|}{0:~               }{3:|}{0:~              }|
-          {0:~                   }{3:|}{0:~               }{3:|}{0:~              }|
-          {0:~                   }{3:|}{0:~               }{3:|}{0:~              }|
-          {0:~                   }{3:|}{0:~               }{3:|}{0:~              }|
-          {0:~                   }{3:|}{0:~               }{3:|}{0:~              }|
+          ^                    {3:│}                {3:│}               |
+          {0:~                   }{3:│}{0:~               }{3:│}{0:~              }|
+          {0:~                   }{3:│}{0:~               }{3:│}{0:~              }|
+          {0:~                   }{3:│}{0:~               }{3:│}{0:~              }|
+          {0:~                   }{3:│}{0:~               }{3:│}{0:~              }|
+          {0:~                   }{3:│}{0:~               }{3:│}{0:~              }|
           {1:[No Name]            }{3:[No Name]        [No Name]      }|
                                                                |
           {0:~                                                    }|
@@ -206,12 +297,12 @@ describe('Screen', function()
         ]])
         insert('hello')
         screen:expect([[
-          hell^o               {3:|}hello           {3:|}hello          |
-          {0:~                   }{3:|}{0:~               }{3:|}{0:~              }|
-          {0:~                   }{3:|}{0:~               }{3:|}{0:~              }|
-          {0:~                   }{3:|}{0:~               }{3:|}{0:~              }|
-          {0:~                   }{3:|}{0:~               }{3:|}{0:~              }|
-          {0:~                   }{3:|}{0:~               }{3:|}{0:~              }|
+          hell^o               {3:│}hello           {3:│}hello          |
+          {0:~                   }{3:│}{0:~               }{3:│}{0:~              }|
+          {0:~                   }{3:│}{0:~               }{3:│}{0:~              }|
+          {0:~                   }{3:│}{0:~               }{3:│}{0:~              }|
+          {0:~                   }{3:│}{0:~               }{3:│}{0:~              }|
+          {0:~                   }{3:│}{0:~               }{3:│}{0:~              }|
           {1:[No Name] [+]        }{3:[No Name] [+]    [No Name] [+]  }|
           hello                                                |
           {0:~                                                    }|
@@ -225,19 +316,19 @@ describe('Screen', function()
     end)
   end)
 
-  describe('tabnew', function()
-    it('creates a new buffer', function()
+  describe('tabs', function()
+    it('tabnew creates a new buffer', function()
       command('sp')
       command('vsp')
       command('vsp')
       insert('hello')
       screen:expect([[
-        hell^o               {3:|}hello           {3:|}hello          |
-        {0:~                   }{3:|}{0:~               }{3:|}{0:~              }|
-        {0:~                   }{3:|}{0:~               }{3:|}{0:~              }|
-        {0:~                   }{3:|}{0:~               }{3:|}{0:~              }|
-        {0:~                   }{3:|}{0:~               }{3:|}{0:~              }|
-        {0:~                   }{3:|}{0:~               }{3:|}{0:~              }|
+        hell^o               {3:│}hello           {3:│}hello          |
+        {0:~                   }{3:│}{0:~               }{3:│}{0:~              }|
+        {0:~                   }{3:│}{0:~               }{3:│}{0:~              }|
+        {0:~                   }{3:│}{0:~               }{3:│}{0:~              }|
+        {0:~                   }{3:│}{0:~               }{3:│}{0:~              }|
+        {0:~                   }{3:│}{0:~               }{3:│}{0:~              }|
         {1:[No Name] [+]        }{3:[No Name] [+]    [No Name] [+]  }|
         hello                                                |
         {0:~                                                    }|
@@ -269,12 +360,12 @@ describe('Screen', function()
       command('tabprevious')
       screen:expect([[
         {2: }{6:4}{2:+ [No Name] }{4: + [No Name] }{3:                         }{4:X}|
-        hell^o               {3:|}hello           {3:|}hello          |
-        {0:~                   }{3:|}{0:~               }{3:|}{0:~              }|
-        {0:~                   }{3:|}{0:~               }{3:|}{0:~              }|
-        {0:~                   }{3:|}{0:~               }{3:|}{0:~              }|
-        {0:~                   }{3:|}{0:~               }{3:|}{0:~              }|
-        {0:~                   }{3:|}{0:~               }{3:|}{0:~              }|
+        hell^o               {3:│}hello           {3:│}hello          |
+        {0:~                   }{3:│}{0:~               }{3:│}{0:~              }|
+        {0:~                   }{3:│}{0:~               }{3:│}{0:~              }|
+        {0:~                   }{3:│}{0:~               }{3:│}{0:~              }|
+        {0:~                   }{3:│}{0:~               }{3:│}{0:~              }|
+        {0:~                   }{3:│}{0:~               }{3:│}{0:~              }|
         {1:[No Name] [+]        }{3:[No Name] [+]    [No Name] [+]  }|
         hello                                                |
         {0:~                                                    }|
@@ -284,6 +375,196 @@ describe('Screen', function()
                                                              |
       ]])
     end)
+
+    it('tabline is redrawn after messages', function()
+      command('tabnew')
+      screen:expect([[
+        {4: [No Name] }{2: [No Name] }{3:                              }{4:X}|
+        ^                                                     |
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+                                                             |
+      ]])
+
+      feed(':echo "'..string.rep('x\\n', 11)..'"<cr>')
+      screen:expect([[
+        {1:                                                     }|
+        x                                                    |
+        x                                                    |
+        x                                                    |
+        x                                                    |
+        x                                                    |
+        x                                                    |
+        x                                                    |
+        x                                                    |
+        x                                                    |
+        x                                                    |
+        x                                                    |
+                                                             |
+        {7:Press ENTER or type command to continue}^              |
+      ]])
+
+      feed('<cr>')
+      screen:expect([[
+        {4: [No Name] }{2: [No Name] }{3:                              }{4:X}|
+        ^                                                     |
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+                                                             |
+      ]])
+
+      feed(':echo "'..string.rep('x\\n', 12)..'"<cr>')
+      screen:expect([[
+        x                                                    |
+        x                                                    |
+        x                                                    |
+        x                                                    |
+        x                                                    |
+        x                                                    |
+        x                                                    |
+        x                                                    |
+        x                                                    |
+        x                                                    |
+        x                                                    |
+        x                                                    |
+                                                             |
+        {7:Press ENTER or type command to continue}^              |
+      ]])
+
+      feed('<cr>')
+      screen:expect([[
+        {4: [No Name] }{2: [No Name] }{3:                              }{4:X}|
+        ^                                                     |
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+                                                             |
+      ]])
+
+    end)
+
+    it('redraws properly with :tab split right after scroll', function()
+      feed('15Ofoo<esc>15Obar<esc>gg')
+
+      command('vsplit')
+      screen:expect([[
+        ^foo                       {3:│}foo                       |
+        foo                       {3:│}foo                       |
+        foo                       {3:│}foo                       |
+        foo                       {3:│}foo                       |
+        foo                       {3:│}foo                       |
+        foo                       {3:│}foo                       |
+        foo                       {3:│}foo                       |
+        foo                       {3:│}foo                       |
+        foo                       {3:│}foo                       |
+        foo                       {3:│}foo                       |
+        foo                       {3:│}foo                       |
+        foo                       {3:│}foo                       |
+        {1:[No Name] [+]              }{3:[No Name] [+]             }|
+                                                             |
+      ]])
+
+      feed('<PageDown>')
+      screen:expect([[
+        ^foo                       {3:│}foo                       |
+        foo                       {3:│}foo                       |
+        foo                       {3:│}foo                       |
+        foo                       {3:│}foo                       |
+        bar                       {3:│}foo                       |
+        bar                       {3:│}foo                       |
+        bar                       {3:│}foo                       |
+        bar                       {3:│}foo                       |
+        bar                       {3:│}foo                       |
+        bar                       {3:│}foo                       |
+        bar                       {3:│}foo                       |
+        bar                       {3:│}foo                       |
+        {1:[No Name] [+]              }{3:[No Name] [+]             }|
+                                                             |
+      ]])
+      command('tab split')
+      screen:expect([[
+        {4: }{5:2}{4:+ [No Name] }{2: + [No Name] }{3:                         }{4:X}|
+        ^foo                                                  |
+        foo                                                  |
+        foo                                                  |
+        foo                                                  |
+        bar                                                  |
+        bar                                                  |
+        bar                                                  |
+        bar                                                  |
+        bar                                                  |
+        bar                                                  |
+        bar                                                  |
+        bar                                                  |
+                                                             |
+      ]])
+    end)
+
+    it('redraws unvisited tab #9152', function()
+      insert('hello')
+      -- create a tab without visiting it
+      command('tabnew|tabnext')
+      screen:expect([[
+        {2: + [No Name] }{4: [No Name] }{3:                            }{4:X}|
+        hell^o                                                |
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+                                                             |
+      ]])
+
+      feed('gT')
+      screen:expect([[
+        {4: + [No Name] }{2: [No Name] }{3:                            }{4:X}|
+        ^                                                     |
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+                                                             |
+      ]])
+    end)
   end)
 
   describe('insert mode', function()
@@ -355,7 +636,8 @@ describe('Screen', function()
       ]])
     end)
 
-    it('execute command with multi-line output', function()
+    it('execute command with multi-line output without msgsep', function()
+      command("set display-=msgsep")
       feed(':ls<cr>')
       screen:expect([[
         {0:~                                                    }|
@@ -375,6 +657,28 @@ describe('Screen', function()
       ]])
       feed('<cr>') --  skip the "Press ENTER..." state or tests will hang
     end)
+
+    it('execute command with multi-line output and with msgsep', function()
+      command("set display+=msgsep")
+      feed(':ls<cr>')
+      screen:expect([[
+                                                             |
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {0:~                                                    }|
+        {1:                                                     }|
+        :ls                                                  |
+          1 %a   "[No Name]"                    line 1       |
+        {7:Press ENTER or type command to continue}^              |
+      ]])
+      feed('<cr>') --  skip the "Press ENTER..." state or tests will hang
+    end)
   end)
 
   describe('scrolling and clearing', function()
@@ -398,12 +702,12 @@ describe('Screen', function()
       command('vsp')
       command('vsp')
       screen:expect([[
-        and                 {3:|}and             {3:|}and            |
-        clearing            {3:|}clearing        {3:|}clearing       |
-        in                  {3:|}in              {3:|}in             |
-        split               {3:|}split           {3:|}split          |
-        windows             {3:|}windows         {3:|}windows        |
-        ^                    {3:|}                {3:|}               |
+        and                 {3:│}and             {3:│}and            |
+        clearing            {3:│}clearing        {3:│}clearing       |
+        in                  {3:│}in              {3:│}in             |
+        split               {3:│}split           {3:│}split          |
+        windows             {3:│}windows         {3:│}windows        |
+        ^                    {3:│}                {3:│}               |
         {1:[No Name] [+]        }{3:[No Name] [+]    [No Name] [+]  }|
         clearing                                             |
         in                                                   |
@@ -418,12 +722,12 @@ describe('Screen', function()
     it('only affects the current scroll region', function()
       feed('6k')
       screen:expect([[
-        ^scrolling           {3:|}and             {3:|}and            |
-        and                 {3:|}clearing        {3:|}clearing       |
-        clearing            {3:|}in              {3:|}in             |
-        in                  {3:|}split           {3:|}split          |
-        split               {3:|}windows         {3:|}windows        |
-        windows             {3:|}                {3:|}               |
+        ^scrolling           {3:│}and             {3:│}and            |
+        and                 {3:│}clearing        {3:│}clearing       |
+        clearing            {3:│}in              {3:│}in             |
+        in                  {3:│}split           {3:│}split          |
+        split               {3:│}windows         {3:│}windows        |
+        windows             {3:│}                {3:│}               |
         {1:[No Name] [+]        }{3:[No Name] [+]    [No Name] [+]  }|
         clearing                                             |
         in                                                   |
@@ -435,12 +739,12 @@ describe('Screen', function()
       ]])
       feed('<c-w>l')
       screen:expect([[
-        scrolling           {3:|}and                 {3:|}and        |
-        and                 {3:|}clearing            {3:|}clearing   |
-        clearing            {3:|}in                  {3:|}in         |
-        in                  {3:|}split               {3:|}split      |
-        split               {3:|}windows             {3:|}windows    |
-        windows             {3:|}^                    {3:|}           |
+        scrolling           {3:│}and                 {3:│}and        |
+        and                 {3:│}clearing            {3:│}clearing   |
+        clearing            {3:│}in                  {3:│}in         |
+        in                  {3:│}split               {3:│}split      |
+        split               {3:│}windows             {3:│}windows    |
+        windows             {3:│}^                    {3:│}           |
         {3:[No Name] [+]        }{1:[No Name] [+]        }{3:<Name] [+] }|
         clearing                                             |
         in                                                   |
@@ -452,12 +756,12 @@ describe('Screen', function()
       ]])
       feed('gg')
       screen:expect([[
-        scrolling           {3:|}^Inserting           {3:|}and        |
-        and                 {3:|}text                {3:|}clearing   |
-        clearing            {3:|}with                {3:|}in         |
-        in                  {3:|}many                {3:|}split      |
-        split               {3:|}lines               {3:|}windows    |
-        windows             {3:|}to                  {3:|}           |
+        scrolling           {3:│}^Inserting           {3:│}and        |
+        and                 {3:│}text                {3:│}clearing   |
+        clearing            {3:│}with                {3:│}in         |
+        in                  {3:│}many                {3:│}split      |
+        split               {3:│}lines               {3:│}windows    |
+        windows             {3:│}to                  {3:│}           |
         {3:[No Name] [+]        }{1:[No Name] [+]        }{3:<Name] [+] }|
         clearing                                             |
         in                                                   |
@@ -469,12 +773,12 @@ describe('Screen', function()
       ]])
       feed('7j')
       screen:expect([[
-        scrolling           {3:|}with                {3:|}and        |
-        and                 {3:|}many                {3:|}clearing   |
-        clearing            {3:|}lines               {3:|}in         |
-        in                  {3:|}to                  {3:|}split      |
-        split               {3:|}test                {3:|}windows    |
-        windows             {3:|}^scrolling           {3:|}           |
+        scrolling           {3:│}with                {3:│}and        |
+        and                 {3:│}many                {3:│}clearing   |
+        clearing            {3:│}lines               {3:│}in         |
+        in                  {3:│}to                  {3:│}split      |
+        split               {3:│}test                {3:│}windows    |
+        windows             {3:│}^scrolling           {3:│}           |
         {3:[No Name] [+]        }{1:[No Name] [+]        }{3:<Name] [+] }|
         clearing                                             |
         in                                                   |
@@ -486,12 +790,12 @@ describe('Screen', function()
       ]])
       feed('2j')
       screen:expect([[
-        scrolling           {3:|}lines               {3:|}and        |
-        and                 {3:|}to                  {3:|}clearing   |
-        clearing            {3:|}test                {3:|}in         |
-        in                  {3:|}scrolling           {3:|}split      |
-        split               {3:|}and                 {3:|}windows    |
-        windows             {3:|}^clearing            {3:|}           |
+        scrolling           {3:│}lines               {3:│}and        |
+        and                 {3:│}to                  {3:│}clearing   |
+        clearing            {3:│}test                {3:│}in         |
+        in                  {3:│}scrolling           {3:│}split      |
+        split               {3:│}and                 {3:│}windows    |
+        windows             {3:│}^clearing            {3:│}           |
         {3:[No Name] [+]        }{1:[No Name] [+]        }{3:<Name] [+] }|
         clearing                                             |
         in                                                   |
@@ -503,12 +807,12 @@ describe('Screen', function()
       ]])
       feed('5k')
       screen:expect([[
-        scrolling           {3:|}^lines               {3:|}and        |
-        and                 {3:|}to                  {3:|}clearing   |
-        clearing            {3:|}test                {3:|}in         |
-        in                  {3:|}scrolling           {3:|}split      |
-        split               {3:|}and                 {3:|}windows    |
-        windows             {3:|}clearing            {3:|}           |
+        scrolling           {3:│}^lines               {3:│}and        |
+        and                 {3:│}to                  {3:│}clearing   |
+        clearing            {3:│}test                {3:│}in         |
+        in                  {3:│}scrolling           {3:│}split      |
+        split               {3:│}and                 {3:│}windows    |
+        windows             {3:│}clearing            {3:│}           |
         {3:[No Name] [+]        }{1:[No Name] [+]        }{3:<Name] [+] }|
         clearing                                             |
         in                                                   |
@@ -520,12 +824,12 @@ describe('Screen', function()
       ]])
       feed('k')
       screen:expect([[
-        scrolling           {3:|}^many                {3:|}and        |
-        and                 {3:|}lines               {3:|}clearing   |
-        clearing            {3:|}to                  {3:|}in         |
-        in                  {3:|}test                {3:|}split      |
-        split               {3:|}scrolling           {3:|}windows    |
-        windows             {3:|}and                 {3:|}           |
+        scrolling           {3:│}^many                {3:│}and        |
+        and                 {3:│}lines               {3:│}clearing   |
+        clearing            {3:│}to                  {3:│}in         |
+        in                  {3:│}test                {3:│}split      |
+        split               {3:│}scrolling           {3:│}windows    |
+        windows             {3:│}and                 {3:│}           |
         {3:[No Name] [+]        }{1:[No Name] [+]        }{3:<Name] [+] }|
         clearing                                             |
         in                                                   |
@@ -557,9 +861,10 @@ describe('Screen', function()
     it('has minimum width/height values', function()
       screen:try_resize(1, 1)
       screen:expect([[
-        {2:-- INS^ERT --}|
-                    |
+        resize^      |
+        {2:-- INSERT -} |
       ]])
+
       feed('<esc>:ls')
       screen:expect([[
         resize      |
@@ -573,6 +878,7 @@ describe('Screen', function()
       command('nnoremap <F1> :echo "TEST"<CR>')
       feed(':ls<CR>')
       screen:expect([[
+                                                             |
         {0:~                                                    }|
         {0:~                                                    }|
         {0:~                                                    }|
@@ -582,8 +888,7 @@ describe('Screen', function()
         {0:~                                                    }|
         {0:~                                                    }|
         {0:~                                                    }|
-        {0:~                                                    }|
-        {0:~                                                    }|
+        {1:                                                     }|
         :ls                                                  |
           1 %a   "[No Name]"                    line 1       |
         {7:Press ENTER or type command to continue}^              |
@@ -607,22 +912,51 @@ describe('Screen', function()
       ]])
     end)
   end)
+
+  -- Regression test for #8357
+  it('does not have artifacts after temporary chars in insert mode', function()
+    command('inoremap jk <esc>')
+    feed('ifooj')
+    screen:expect([[
+      foo^j                                                 |
+      {0:~                                                    }|
+      {0:~                                                    }|
+      {0:~                                                    }|
+      {0:~                                                    }|
+      {0:~                                                    }|
+      {0:~                                                    }|
+      {0:~                                                    }|
+      {0:~                                                    }|
+      {0:~                                                    }|
+      {0:~                                                    }|
+      {0:~                                                    }|
+      {0:~                                                    }|
+      {2:-- INSERT --}                                         |
+    ]])
+    feed('k')
+    screen:expect([[
+      fo^o                                                  |
+      {0:~                                                    }|
+      {0:~                                                    }|
+      {0:~                                                    }|
+      {0:~                                                    }|
+      {0:~                                                    }|
+      {0:~                                                    }|
+      {0:~                                                    }|
+      {0:~                                                    }|
+      {0:~                                                    }|
+      {0:~                                                    }|
+      {0:~                                                    }|
+      {0:~                                                    }|
+                                                           |
+    ]])
+  end)
+end
+
+describe("Screen (char-based)", function()
+  screen_tests(false)
 end)
 
-describe('nvim_ui_attach()', function()
-  before_each(function()
-    clear()
-  end)
-  it('handles very large width/height #2180', function()
-    local screen = Screen.new(999, 999)
-    screen:attach()
-    eq(999, eval('&lines'))
-    eq(999, eval('&columns'))
-  end)
-  it('invalid option returns error', function()
-    local screen = Screen.new()
-    local status, rv = pcall(function() screen:attach({foo={'foo'}}) end)
-    eq(false, status)
-    eq('No such ui option', rv:match("No such .*"))
-  end)
+describe("Screen (line-based)", function()
+  screen_tests(true)
 end)
